package dont.panic.renewed.activity;






import java.util.ArrayList;
import java.util.Timer;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import dont.panic.renewed.R;
import dont.panic.renewed.adapters.MapAndActionFragmentAdapter;
import dont.panic.renewed.fragments.CustomMapFragment;
import dont.panic.renewed.fragments.ZoneFragment;
import dont.panic.renewed.interfaces.ActionPerformedListenerInterface;
import dont.panic.renewed.interfaces.LocationlistenerInterface;
import dont.panic.renewed.interfaces.NetworkMessageInterface;
import dont.panic.renewed.models.Civilian;
import dont.panic.renewed.models.Map;
import dont.panic.renewed.models.Player;
import dont.panic.renewed.models.ReflectionData;
import dont.panic.renewed.models.ReflectionQuestions;
import dont.panic.renewed.models.Zone;
import dont.panic.renewed.network.GameClient;
import dont.panic.renewed.utils.AllPlayerMarkers;
import dont.panic.renewed.utils.DebugConfig;
import dont.panic.renewed.utils.EquipAndBlockNames;
import dont.panic.renewed.utils.GlobalZoneStatusHelper;
import dont.panic.renewed.utils.RolesNamesAndBenefis;
import android.opengl.EGLDisplay;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Looper;
import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.ActionBar.TabListener;
import android.app.AlarmManager;
import android.app.AlertDialog;
import android.app.FragmentTransaction;
import android.app.PendingIntent;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v4.app.FragmentActivity;
import android.support.v4.view.ViewPager;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

/**
 * The main activity, this application only uses this one activity for the entire game and fragments for the map and zone GUI
 * 
 */

public class MainActivity extends FragmentActivity implements TabListener, NetworkMessageInterface, LocationlistenerInterface, ActionPerformedListenerInterface {
	
	private MapAndActionFragmentAdapter fragmentAdapter;
	private ViewPager viewPager;
	private GameClient client;
	private Thread t;
	private int hearthBeatDelayTimer;
	private boolean useHearthBeat;

	/**
	 * Default onCreate function in android application.
	 * This sets up the fragment adapter which is needed to use fragments and also setup the player object.
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		ActionBar actionbar = getActionBar();
		fragmentAdapter = new MapAndActionFragmentAdapter(getSupportFragmentManager());
		actionbar.setNavigationMode(2);
		hearthBeatDelayTimer = 0;
		useHearthBeat = true;
		
//		Player.getInstance().setNickname("Alex");
//		Player.getInstance().setRole(RolesNamesAndBenefis.VOLUNTEER);
		Player.getInstance().setUpTestEquipmentData();
		if(Player.getInstance().getNickname() == null){
			SharedPreferences settings = getSharedPreferences("Prefs", 0);
			if(settings.contains("nickname")){
				Player.getInstance().setNickname(settings.getString("nickname", ""));
				Player.getInstance().setRole(RolesNamesAndBenefis.getRoleFromList(settings.getInt("roleInt", 0)));
			}
		}
		
        viewPager = (ViewPager) findViewById(R.id.activity_main_pager);
        
        viewPager.setAdapter(fragmentAdapter);
        
        viewPager.setOnPageChangeListener(new ViewPager.SimpleOnPageChangeListener() {
            @Override
            public void onPageSelected(int position) {
                // When swiping between different app sections, select the corresponding tab.
                // We can also use ActionBar.Tab#select() to do this if we have a reference to the
                // Tab.
                getActionBar().setSelectedNavigationItem(position);
            }
        });
        
        // For each of the sections/fragments in the adapter, add a tab to the action bar.
        for (int i = 0; i < fragmentAdapter.getCount(); i++) {
        	actionbar.addTab(
        			actionbar.newTab()
                            .setText(fragmentAdapter.getPageTitle(i))
                            .setTabListener(this));
        }
        viewPager.requestTransparentRegion(viewPager);
//        DebugConfig.getInstance().useNetwork = false;
        if(DebugConfig.getInstance().useNetwork == true){
        	setUpClient();
        }
        
		
	}

	/**
	 * 
	 * @see android.app.ActionBar.TabListener#onTabReselected(android.app.ActionBar.Tab, android.app.FragmentTransaction)
	 */
	@Override
	public void onTabReselected(Tab tab, FragmentTransaction ft) {
		// TODO Auto-generated method stub
		
	}
	
	/**
	 * Set up so the user can change fragment by pressing the buttons at the top bar
	 */

	@Override
	public void onTabSelected(Tab tab, FragmentTransaction ft) {
		viewPager.setCurrentItem(tab.getPosition());
	}

	@Override
	public void onTabUnselected(Tab tab, FragmentTransaction ft) {
		// TODO Auto-generated method stub
	
	}
	
	/**
	 * this method establish the connection with the server.
	 */
	public void setUpClient(){
//		client = new GameClient("129.241.103.221", this);
		client = new GameClient("129.241.103.221", this);
		t = new Thread(new Runnable() {
			
			@Override
			public void run() {
				try{
					client.run();
				} catch(RuntimeException ex) {
					runOnUiThread(new Runnable() {
					     public void run() {
//					    	 failToConnectToServer();
					    }
					});	
				}
				
			}
		});
		t.start();
	}

	/**
	 * This function handle the various network message events
	 * @see dont.panic.renewed.interfaces.NetworkMessageInterface#onMessageRecieved(java.lang.String)
	 */
	@Override
	public void onMessageRecieved(String message) {
		if(message != null){
			final String[] rawMesaage =message.split("\\|");
			String header = rawMesaage[0];
			String sender = rawMesaage[1];
			if(header.equalsIgnoreCase("map")){
				setUpMap(rawMesaage[1]);
				startHearthBeatMonitor();
			}
			
			else if(header.equalsIgnoreCase("msg")){
			 String msg = rawMesaage[1];
			}
			else if(header.equalsIgnoreCase("loc")){
				String rawLoc = rawMesaage[2];
				String loc[] = rawLoc.split("~");
				AllPlayerMarkers.getInstance().updatePosition(sender, Double.valueOf(loc[0]), Double.valueOf(loc[1]));
				
				
			}
			else if(header.equalsIgnoreCase("hearthBeat")){
				runOnUiThread(new Runnable() {
				     public void run() {
				    	hearthBeatDelayTimer = 0;
				    }
				});
			}
			
			else if(header.equalsIgnoreCase("join")){
				if (!sender.equalsIgnoreCase(Player.getInstance().getNickname())) {
					AllPlayerMarkers.getInstance().addNewPlayermarker(sender);
				}
			}
			
			else if(header.equalsIgnoreCase("block")){
				if (!sender.equalsIgnoreCase(Player.getInstance().getNickname())) {
					String rawBlock = rawMesaage[2];
					String block[] = rawBlock.split("~");
					CustomMapFragment customMapFragment = (CustomMapFragment)fragmentAdapter.getItem(0);
					for(Zone zone: customMapFragment.getZoneList()){
						if(zone.getName().equalsIgnoreCase(block[0])){
							zone.unlockBlockade(block[1]);
						}
					}
				
					if(GlobalZoneStatusHelper.getInstance().isInZone() && 
							GlobalZoneStatusHelper.getInstance().getCurrentZone().getName().equalsIgnoreCase(block[0])){
						final ZoneFragment zoneFrag  = (ZoneFragment)fragmentAdapter.getItem(1);
						runOnUiThread(new Runnable() {
						     public void run() {
						    	 zoneFrag.onZoneEnter();
						    }
						});
						
					}
				}
			}
			
			else if(header.equalsIgnoreCase("civSingle")){
				if (!sender.equalsIgnoreCase(Player.getInstance().getNickname())) {
					String rawCivAction = rawMesaage[2];
					String civAction[] = rawCivAction.split("~");
					CustomMapFragment customMapFragment = (CustomMapFragment)fragmentAdapter.getItem(0);
					for(Zone zone: customMapFragment.getZoneList()){
						if(zone.getName().equalsIgnoreCase(civAction[0])){
							zone.performActionOnCivilian(civAction[1],civAction[2]);
						}
					}
					
					if(GlobalZoneStatusHelper.getInstance().isInZone() && 
							GlobalZoneStatusHelper.getInstance().getCurrentZone().getName().equalsIgnoreCase(civAction[0])){
						final ZoneFragment zoneFrag  = (ZoneFragment)fragmentAdapter.getItem(1);
						runOnUiThread(new Runnable() {
						     public void run() {
						    	 zoneFrag.updateCiviliansStatus();
						    }
						});
					}
				}
			}
			
			else if(header.equalsIgnoreCase("civAll")){
				if (!sender.equalsIgnoreCase(Player.getInstance().getNickname())) {
					String rawCivAction = rawMesaage[2];
					String civAction[] = rawCivAction.split("~");
					CustomMapFragment customMapFragment = (CustomMapFragment)fragmentAdapter.getItem(0);
					for(Zone zone: customMapFragment.getZoneList()){
						if(zone.getName().equalsIgnoreCase(civAction[0])){
							zone.performActionOnAllCivilian(civAction[1]);
						}
					}
					
					if(GlobalZoneStatusHelper.getInstance().isInZone() && 
							GlobalZoneStatusHelper.getInstance().getCurrentZone().getName().equalsIgnoreCase(civAction[0])){
						final ZoneFragment zoneFrag  = (ZoneFragment)fragmentAdapter.getItem(1);
						runOnUiThread(new Runnable() {
						     public void run() {
						    	 zoneFrag.updateCiviliansStatus();
						    }
						});
					}
				}
			}
			
			else if(header.equalsIgnoreCase("tick")){
				CustomMapFragment customMapFragment = (CustomMapFragment)fragmentAdapter.getItem(0);
				for(Zone zone : customMapFragment.getZoneList()){
					zone.gameTick();	
				}
				if(GlobalZoneStatusHelper.getInstance().isInZone()){
					if(!GlobalZoneStatusHelper.getInstance().getCurrentZone().isLocked()){
						final ZoneFragment zoneFrag  = (ZoneFragment)fragmentAdapter.getItem(1);
						if(zoneFrag != null){		
							
							runOnUiThread(new Runnable() {
							     public void run() {
							    	 zoneFrag.updateCiviliansStatus();
							    }
							});
						}
					}
				}
			}
			
			else if(header.equalsIgnoreCase("panicWave")){
				CustomMapFragment customMapFragment = (CustomMapFragment)fragmentAdapter.getItem(0);
				for(Zone zone : customMapFragment.getZoneList()){
					zone.panickWave(10);	
				}
			}
			
			else if(header.equalsIgnoreCase("recoverEquipment")){
				Gson gson = new Gson();
				Player.getInstance().recoverEquipment(gson.fromJson(rawMesaage[2], ArrayList.class ));
			}
			else if(header.equalsIgnoreCase("normalPause")){
				final int duration = Integer.parseInt(sender);
				
				runOnUiThread(new Runnable() {
				     public void run() {
				    	 gamePause(duration);
				    }
				});


			}
			else if(header.equalsIgnoreCase("gameFinish")){		
				
				runOnUiThread(new Runnable() {
				     public void run() {
				    	 onGameComplete();
				    }
				});
			}
			
			else if(header.equalsIgnoreCase("drawLines")){
				final CustomMapFragment customMapFragment = (CustomMapFragment)fragmentAdapter.getItem(0);
				runOnUiThread(new Runnable() {
				     public void run() {
				    	customMapFragment.drawlines();
				    }
				});
			}

			else if(header.equalsIgnoreCase("reflectionData")){
				runOnUiThread(new Runnable() {
				     public void run() {
				    	 setUpReflectionData(rawMesaage[1]);
				    }
				});
			}
			
			
		}
	}
	
	/**
	 * This function is called when a sucessfull connection have been made to the server. It accepts a Json string with the map
	 * detail and create the object.
	 */
	private void setUpMap(String json){
		Gson gson = new Gson();
		
		CustomMapFragment customMapFragment = (CustomMapFragment)fragmentAdapter.getItem(0);
		customMapFragment.serverMap= gson.fromJson(json, Map.class);
		customMapFragment.setUpZoneMarkersFromNetworkMap();
		
		customMapFragment.setLocationObserver(this);
		ZoneFragment zoneFrag  = (ZoneFragment)fragmentAdapter.getItem(1);
		zoneFrag.setActionPerformedObserver(this);
		
		for(Zone zone : customMapFragment.getZoneList()){
			zone.createEquipmentFromServer();
		}
	}
	
	/**
	 * This method is called when the player recieve the reflection data previously entered and submitted 
	 * to the server.
	 * @param json The string that contain all the answers
	 */
	private void setUpReflectionData(String json){
		Gson gson = new Gson();
		Player.getInstance().setReflectionData(gson.fromJson(json, ReflectionData.class));
		setUpReviewState();
		useHearthBeat = false;
	}
	

	/**
	 * Function that allert the user that the connection to the server has been lost
	 * @see dont.panic.renewed.interfaces.NetworkMessageInterface#onNetworkConnectionLost(java.lang.String)
	 */
	@Override
	public void onNetworkConnectionLost(final String message) {
		// TODO Auto-generated method stub
		runOnUiThread(new Runnable() {
		     public void run() {
		    	 playerDisconnect(message);
		    }
		});	
		
	}
	
	/**
	 * This function is called when the client havent recieved a hearthbeat message within 5 second
	 * @param message not in use in this prototype, but will allow the server to select what message to display to the user
	 */
	public void playerDisconnect(String message){
		 new AlertDialog.Builder(this)
	        .setIcon(android.R.drawable.ic_dialog_alert)
	        .setTitle("Connection lost!!!!!")
	        .setMessage("Please press restart to reconnect")
	        .setPositiveButton("restart", new DialogInterface.OnClickListener()
	    {
	        @Override
	        public void onClick(DialogInterface dialog, int which) {
	        	restartApplication();
	        }

	    })
	    .show();
	}

	/**
	 * This function will run when the build inn location listener detect movement. The new coordinates are sendt to the server so that
	 * other players can get theupdated location
	 * @see dont.panic.renewed.interfaces.LocationlistenerInterface#onLocationChanged(java.lang.String, java.lang.String)
	 */
	@Override
	public void onLocationChanged(final String lat, final String lng) {
	 if(DebugConfig.getInstance().useNetwork == true){
			Thread tr = new Thread(new Runnable() {	
				@Override
				public void run() {
					// TODO Auto-generated method stub
					client.sendPlayerPositionMessage(lat, lng);
				}
			});	
			tr.start();
	 }
		
	}
	
	/**
	 * This function is called when the mapFragment notice the user has entered a new zone.
	 * It will send a network message to the server so the manager GUI can update the current player location.
	 */
	@Override
	public void onZoneEntered(final String zoneName) {
		 if(DebugConfig.getInstance().useNetwork == true){
				Thread tr = new Thread(new Runnable() {	
					@Override
					public void run() {
						// TODO Auto-generated method stub
						client.sendZoneEntered(zoneName);
					}
				});	
				tr.start();
		 }
		
	}

	/**
	 * This function is called when the mapFragment notice the user has left a zone.
	 * It will send a network message to the server so the manager GUI can update the last zone location of the player.
	 */
	@Override
	public void onZoneLeft() {
		 if(DebugConfig.getInstance().useNetwork == true){
				Thread tr = new Thread(new Runnable() {	
					@Override
					public void run() {
						// TODO Auto-generated method stub
						client.sendZoneLeft();
					}
				});	
				tr.start();
		 }
		
	}



	/**
	 * This function is called when the player sucessfully remove a blockade from a zone.The message will be sent to the server so
	 * that other players can get the updated state as well.
	 * @see dont.panic.renewed.interfaces.ActionPerformedListenerInterface#onZoneUnlocked(java.lang.String, java.lang.String)
	 */
	@Override
	public void onZoneUnlocked(final String zoneName, final String blockadeName) {
		 if(DebugConfig.getInstance().useNetwork == true){
			Thread tr = new Thread(new Runnable() {	
				@Override
				public void run() {
					// TODO Auto-generated method stub
					client.sendPlayerZoneBlockadeActionDone(zoneName, blockadeName);


				}
			});	
			tr.start();
		 }
		
	}

	/**
	 * This function is called when an item is used within a zone. A message will be sent to the server so that the other players get the 
	 * updated state
	 * @see dont.panic.renewed.interfaces.ActionPerformedListenerInterface#onCivilianItemUsed(java.lang.String, java.lang.String)
	 */
	@Override
	public void onCivilianItemUsed(final String zoneName, final String itemName) {

		Thread tr = new Thread(new Runnable() {	
			@Override
			public void run() {
				// TODO Auto-generated method stub
				client.sendPlayerZoneCivlianItemActionDone(zoneName, itemName);
			}
		});	
		tr.start();
		
	}
	/**
	 * This function is called when an action is perormed within a zone. A message will be sent to the server so that the other players get the 
	 * updated state
	 * @see dont.panic.renewed.interfaces.ActionPerformedListenerInterface#onCivilianAction(java.lang.String, java.lang.String, java.lang.String)
	 */

	@Override
	public void onCivilianAction(final String zoneName, final String civilianName, final String action) {
		Thread tr = new Thread(new Runnable() {	
			@Override
			public void run() {
				// TODO Auto-generated method stub
				client.sendPlayerZoneCivlianActionDone(zoneName, civilianName, action);
			}
		});	
		tr.start();
		
	}
	/**
	 * This function is called when an item is used . A message will be sent to the server so the server can update the player state
	 * so if the player disconnect, he can reconnect and recover the correct state
	 * @see dont.panic.renewed.interfaces.ActionPerformedListenerInterface#onEquipmentUsed(java.lang.String)
	 */

	@Override
	public void onEquipmentUsed(final String equipmentName) {
		Thread tr = new Thread(new Runnable() {	
			@Override
			public void run() {
				// TODO Auto-generated method stub
				client.sendEquipmentUsedAction(equipmentName);
			}
		});	
		tr.start();
		
		
	}
	/**
	 * This function is called when an item is picked up within a zone. A message will be sent to the server so that the other players get the 
	 * updated state
	 * @see dont.panic.renewed.interfaces.ActionPerformedListenerInterface#onEquipmentPickup(java.lang.String)
	 */

	@Override
	public void onEquipmentPickup(final String equipmentName) {
		Thread tr = new Thread(new Runnable() {	
			@Override
			public void run() {
				// TODO Auto-generated method stub
				client.sendEquipmentPickUpAction(equipmentName);
			}
		});	
		tr.start();
		
	}
	
	
	/**
	 * This function is launch a dialog window when the user presses the back button when no dialogwindow is up, 
	 * asking if the player wish to quit the application. 
	 * @see android.support.v4.app.FragmentActivity#onBackPressed()
	 */
	
	@Override
	public void onBackPressed() {
	    new AlertDialog.Builder(this)
	        .setIcon(android.R.drawable.ic_dialog_alert)
	        .setTitle("Closing Activity")
	        .setMessage("Are you sure you want to close this activity?")
	        .setPositiveButton("Yes", new DialogInterface.OnClickListener()
	    {
	        @Override
	        public void onClick(DialogInterface dialog, int which) {
	        	client.closeConnection();
	            finish();   	//marks the app ready to be closed
	            android.os.Process.killProcess(android.os.Process.myPid()); //force the application to be killed immediately.
	        }

	    })
	    .setNegativeButton("No", null)
	    .show();
	}
	
	
	/**
	 * This function is called when the server want the pause the game and trigger the dialogwindow to gather the player emotion
	 * @param duration The duration the pause should last in ms
	 */
	public void gamePause(int duration){
		

		final AlertDialog alertDialog = new AlertDialog.Builder(this).create();  
		alertDialog.setTitle("Game is paused");  
		alertDialog.setMessage("00:??");
		
	
		LayoutInflater inflater = getLayoutInflater();
		LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.dialog_pause, null);
		
		alertDialog.setView(lin);
		alertDialog.show();  
		alertDialog.setCanceledOnTouchOutside(false);
		alertDialog.setCancelable(false);
		
			new CountDownTimer(duration, 1000) {
		    @Override
		    public void onTick(long millisUntilFinished) {
		       alertDialog.setMessage("00:"+ (millisUntilFinished/1000));
		    }
	
		    @Override
		    public void onFinish() {
		    	EditText emotion = (EditText) alertDialog.findViewById(R.id.dialog_pause_emotion_answer);
		    	String answer = emotion.getText().toString().replace("|", "");  //make sure the player doesnt enter symbols which are used by the system.
		    	client.sendPlayerEmotion(answer);
		        alertDialog.dismiss();
		    }
		}.start();
	}
	
	/**
	 * This function is called when the game is done -> when the entire game session time has passed.
	 * 
	 */
	public void onGameComplete(){
		if(!Player.getInstance().getRole().equalsIgnoreCase(RolesNamesAndBenefis.MANAGER)){
			reflectionQuestionOne();
		}
		else{
			useHearthBeat = false;
		}
		
	}
	
	/**
	 * This message is called when the last reflection question is answered,
	 * and will prepare and send the reflection answers to the server.
	 */
	private void onReflectionQuestionDone(){
		client.sendReflectionAnswers(ReflectionQuestions.getInstance().getAnswersFormatedForServer());
	}
	
	/**
	 * The function is called when the application recieve confirmation that the server has recieved the reflection answers.
	 * The review fragment will be created and loaded.
	 */
	public void setUpReviewState(){
		fragmentAdapter.addReviewFragment();
		ActionBar actionBar = getActionBar(); 
		actionBar.addTab(
    			actionBar.newTab()
                        .setText("Review")
                        .setTabListener(this));
		
		ZoneFragment zoneFrag  = (ZoneFragment)fragmentAdapter.getItem(1);
		zoneFrag.showZoneHistory();
		CustomMapFragment mapFrag = (CustomMapFragment)fragmentAdapter.getItem(0);
		mapFrag.stopDrawing();
		
	}
	
	/**
	 * Create the dialogwindow and display reflection question one.
	 */
	public void reflectionQuestionOne(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(getString(R.string.reflection_question_one))
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionOne( answ.getText().toString());		        	  
		        	   
		        	   reflectionQuestionTwo();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question two.
	 */
	public void reflectionQuestionTwo(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_two)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionTwo( answ.getText().toString());		
		        	   reflectionQuestionThree();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question three.
	 */
	public void reflectionQuestionThree(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_three)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionThree( answ.getText().toString());		
		        	   reflectionQuestionFour();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question four.
	 */
	public void reflectionQuestionFour(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_four)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionFour( answ.getText().toString());		
		        	   reflectionQuestionFive();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question five.
	 */
	public void reflectionQuestionFive(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_five)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionFive( answ.getText().toString());		
		        	   reflectionQuestionSix();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question six.
	 */
	public void reflectionQuestionSix(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_six)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionSix( answ.getText().toString());		
		        	   reflectionQuestionSeven();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question seven.
	 */
	public void reflectionQuestionSeven(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_seven)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionSeven( answ.getText().toString());		
		        	   reflectionQuestionEight();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question eight.
	 */
	public void reflectionQuestionEight(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_eight)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionEight( answ.getText().toString());		
		        	   reflectionQuestionNine();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question nine
	 */
	public void reflectionQuestionNine(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_nine)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionNine( answ.getText().toString());		
		        	   reflectionQuestionTen();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question ten.
	 */
	public void reflectionQuestionTen(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_ten)
		       .setCancelable(false)
		       .setPositiveButton("Next", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                //send answer
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionTen( answ.getText().toString());		
		        	   reflectionQuestionEleven();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
	}
	
	/**
	 * Create the dialogwindow and display reflection question eleven.
	 */
	public void reflectionQuestionEleven(){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		LayoutInflater inflater = getLayoutInflater();
		final LinearLayout lin = (LinearLayout) inflater.inflate(R.layout.reuseable_dialog_view_for_question_and_answer, null);
		
		builder.setMessage(R.string.reflection_question_eleven)
		       .setCancelable(false)
		       .setPositiveButton("Finish", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		        	   EditText answ = (EditText) lin.findViewById(R.id.reusable_dialog_view_answer);
		        	   ReflectionQuestions.getInstance().setQuestionEleven( answ.getText().toString());		
		        	   onReflectionQuestionDone();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.setView(lin);
		alert.show();
		
	}
	
	/**
	 * Start the thread which will count the second since the last hearthbeat message
	 * if the time exceed 5 second, it will call the function which will let the player know that the connection has been lost
	 */
	public void startHearthBeatMonitor(){
		final Thread t = new Thread(new Runnable() {
			
			@Override
			public void run() {
				try {
					while(useHearthBeat){
						if(hearthBeatDelayTimer > 5){
							onNetworkConnectionLost("test hearthbeat");
							break;
						}
						hearthBeatDelayTimer ++;
						Thread.sleep(1000);
					}
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
		});
		t.start();
		
	}
	
	/**
	 * This function is called when the player presses "re-connect" button on the "connection-lost" dialogwindow.
	 * It will request the android system to force restart the application.
	 */
	public void restartApplication(){
		AlarmManager alm = (AlarmManager) this.getSystemService(Context.ALARM_SERVICE);
		alm.set(AlarmManager.RTC, System.currentTimeMillis() + 1000, PendingIntent.getActivity(this, 0, new Intent(this, this.getClass()), 0));
		android.os.Process.killProcess(android.os.Process.myPid());
	}
	
	/**
	 * This function will create the dialogwindow which will let the player know that the application failed to connect to the server
	 */
	public void failToConnectToServer(){
		   new AlertDialog.Builder(this)
	        .setIcon(android.R.drawable.ic_dialog_alert)
	        .setCancelable(false)
	        .setTitle("Could not connect to the server")
	        .setMessage("Make sure your connected to a network, and restart the application")
	        .setPositiveButton("Close application", new DialogInterface.OnClickListener()
	        
	        
	    {
	        @Override
	        public void onClick(DialogInterface dialog, int which) {
//	        	client.closeConnection();
	            finish();   	//marks the app ready to be closed
	            android.os.Process.killProcess(android.os.Process.myPid()); //force the application to be killed immediately.
	        }

	    })
	    .show();
	}

	/**
	 * This function is called by the zoneFragment when the player start using an equipment. The function
	 * will alert the server so the manager application can display it to the manager.
	 */
	@Override
	public void onCivilianItemStartUsing(final String zoneName, final String itemName) {
		Thread tr = new Thread(new Runnable() {	
			@Override
			public void run() {
				client.sendPlayerZoneCivlianItemActionStarted(zoneName, itemName);
			}
		});	
		tr.start();
		
	}

	/**
	 * This function is called by the zoneFragment when the player start using an action. The function
	 * will alert the server so the manager application can display it to the manager.
	 */
	@Override
	public void onCivilianActionStartUsing(final String zoneName,
			final String civilianName, final String action) {
		Thread tr = new Thread(new Runnable() {	
			@Override
			public void run() {
				client.sendPlayerZoneCivlianActionStarted(zoneName, civilianName, action);
			}
		});	
		tr.start();
		
	}

	/**
	 * This function is called by the zoneFragment when the player start unlocking a zone. The function
	 * will alert the server so the manager application can display it to the manager.
	 */
	@Override
	public void onZoneStartUnlocking(final String equipName) {
		Thread tr = new Thread(new Runnable() {	
			@Override
			public void run() {
				client.sendPlayerZoneBlockadeActionStarted(equipName);
			}
		});	
		tr.start();
		
		
	}

	/**
	 * This function is called by the network client if it fails to connect to the server
	 */
	@Override
	public void onNetworkConectionFail(String message) {
		runOnUiThread(new Runnable() {
		     public void run() {
		    	 failToConnectToServer();
		    }
		});	
		
	}



}
