package dont.panic.renewed.fragments;

import java.util.ArrayList;
import java.util.Iterator;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener;
import com.google.android.gms.maps.LocationSource.OnLocationChangedListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;


import com.google.android.gms.maps.model.Polyline;
import com.google.android.gms.maps.model.PolylineOptions;
import com.google.maps.android.ui.IconGenerator;

import dont.panic.renewed.R;
import dont.panic.renewed.interfaces.LocationlistenerInterface;
import dont.panic.renewed.models.Map;
import dont.panic.renewed.models.PlayerMarker;
import dont.panic.renewed.models.Zone;
import dont.panic.renewed.utils.AllPlayerMarkers;
import dont.panic.renewed.utils.DebugConfig;
import dont.panic.renewed.utils.GlobalZoneStatusHelper;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.location.Location;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.Toast;


/**
 * MapFragment is the fragment that displays the map
 */
public class CustomMapFragment extends Fragment implements OnMyLocationChangeListener{
	private SupportMapFragment fragment;
	private GoogleMap map;
	private ArrayList<Zone> zoneList;
	public Map serverMap;
	private Thread mapUpdateTimer;
	private int mapUpdateCount = 0;
	private LocationlistenerInterface locationObserver;
	private boolean shouldDraw = true;
	private boolean shouldValidatePosition = false;
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		// TODO Auto-generated method stub
		View rootView = inflater.inflate(R.layout.test_map_to_check_if_api_is_working,container,false );		
		mapUpdateTimer = new Thread(){
			@Override
			public void run() {
				try{
					sleep(5000); //enable a delay before we start drawing the map, this is incase of slow connection
					while(true){
						sleep(1000);
						if(getActivity() != null){
							
							getActivity().runOnUiThread(new Runnable() {
								@Override
								public void run() {
								//	testChangeMakerColor();
									updateMap();
								}
							});
						}
					}
				}catch (InterruptedException e) {
		            e.printStackTrace();
		        }
			}
		};
		mapUpdateTimer.start();

		
		Thread checkLocationTimer = new  Thread(){
			@Override
			public void run() {
					while(shouldDraw){
						try {
							sleep(700);
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						if(getActivity() != null){
							shouldValidatePosition = true;
						}
					}
			}
		};
		checkLocationTimer.start();
		
		
		return rootView;
	}
	
	/**
	 * Set locationlistener so that the object will be notified when the phone has detected a new location change
	 * @param ob the object that want to be notified
	 */
	public void setLocationObserver(LocationlistenerInterface ob){
		this.locationObserver = ob;
	}
	
	/**
	 * 
	 * @see android.support.v4.app.Fragment#onActivityCreated(android.os.Bundle)
	 */
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		FragmentManager fm = getChildFragmentManager();
		fragment = (SupportMapFragment) fm.findFragmentById(R.id.map);
		if (fragment == null) {
			fragment = SupportMapFragment.newInstance();
			fm.beginTransaction().replace(R.id.map, fragment).commit();
			
		}
	}
	
	/**
	 * This function does some required tweak on the map handling as it wont work "out of the box".
	 * The google map object can only be retrieved using async call, this can cause issue if the object is attempted to be used before
	 * it has acually been initialise. This should fix that problem.
	 * @see android.support.v4.app.Fragment#onResume()
	 */
	@Override
	public void onResume() {
		super.onResume();
		if (map == null) {
			map = fragment.getMap();
			setUpZoneMarkers();
			map.setOnMyLocationChangeListener(this);
		}
		map.setMyLocationEnabled(true);
		
	}
	
	/**
	 * This function draw the zone markers on the map
	 */
	private void setUpZoneMarkers(){		
			if(zoneList != null){
			
			IconGenerator iconGen = new IconGenerator(getActivity());
			iconGen.setStyle(IconGenerator.STYLE_GREEN);
			for (Zone zone : zoneList) {
				if(zone.isNormalZone()){
					MarkerOptions marker = new MarkerOptions().position(new LatLng(zone.getLatitude(), zone.getLongitude()));
					marker.icon(BitmapDescriptorFactory.fromBitmap(iconGen.makeIcon(zone.getName())));
					marker.alpha(0.6f);
					map.addMarker(marker);
					zone.setMarker(marker);
				}
				else{
					MarkerOptions marker = new MarkerOptions().position(new LatLng(zone.getLatitude(), zone.getLongitude()));
					map.addMarker(marker);
					zone.setMarker(marker);
				}
			}
		
//			updateMap();
			}
	}
	
	/**
	 * This function is called when the network handler detect that the game state has been sent to the client
	 */
	public void setUpZoneMarkersFromNetworkMap(){
		zoneList = serverMap.getZones();
		
	}
	
	
	/**
	 * This function is called when new location is detected. It will check if the player new location is within a zone or not and if so
	 * it will allow the player to interact with the Zone elements
	 * @see com.google.android.gms.maps.GoogleMap.OnMyLocationChangeListener#onMyLocationChange(android.location.Location)
	 */
	@Override
	public void onMyLocationChange(Location location) {

		if( locationObserver != null && shouldDraw){
			locationObserver.onLocationChanged(String.valueOf(location.getLatitude()), String.valueOf(location.getLongitude()));
		}
		if(zoneList != null && shouldDraw && shouldValidatePosition){
			shouldValidatePosition = false;
			boolean inAnyZone = false;
			for (Zone zone : zoneList) {
				if(zone.isWithinZone(location.getLatitude(), location.getLongitude())){
					if(GlobalZoneStatusHelper.getInstance().isInZone() == false || GlobalZoneStatusHelper.getInstance().getCurrentZone() != zone){
						GlobalZoneStatusHelper.getInstance().setInZone(false); // clear the view of the last zone
						
						 new AlertDialog.Builder(getActivity()).setMessage(("You have entered "+zone.getName() )).show().setCanceledOnTouchOutside(true);;
						
						GlobalZoneStatusHelper.getInstance().setCurrentZone(zone);
						GlobalZoneStatusHelper.getInstance().setInZone(true);
						if( locationObserver != null){
							locationObserver.onZoneEntered(zone.getName());
						}
					}
					inAnyZone = true;
					break;
				}
			}
			if(inAnyZone == false && GlobalZoneStatusHelper.getInstance().isInZone() == true){
				GlobalZoneStatusHelper.getInstance().setInZone(false);
				if( locationObserver != null){
					locationObserver.onZoneLeft();
				}
				new AlertDialog.Builder(getActivity()).setMessage(("You have left" + GlobalZoneStatusHelper.getInstance().getCurrentZone().getName()) ).show().setCanceledOnTouchOutside(true);;
			}
		}
		
	}

	/**
	 * This is just a test function used by us to try different marker colors. Its not used beside debugging
	 */
	
	public void testChangeMakerColor(){
		IconGenerator iconGen = new IconGenerator(getActivity());
		iconGen.setStyle(IconGenerator.STYLE_ORANGE);
		map.clear();
		for (Zone zone : zoneList) {
			MarkerOptions marker = zone.getMarker();
			if(zone.isNormalZone())
				marker.icon(BitmapDescriptorFactory.fromBitmap(iconGen.makeIcon(zone.getName())));
			map.addMarker(marker);
		}
		
		IconGenerator iconGen2 = new IconGenerator(getActivity());
		iconGen2.setStyle(IconGenerator.STYLE_BLUE);
		for(PlayerMarker pl : AllPlayerMarkers.getInstance().getMarkerList()){
			MarkerOptions mark = new MarkerOptions();
			mark.position(new LatLng(pl.getLatitude(), pl.getLongitude()));
			mark.icon(BitmapDescriptorFactory.fromBitmap(iconGen2.makeIcon(pl.getNickname())));
			map.addMarker(mark);
		}
	}

	/**
	 * This function is called at regular intervals so that team mates location is redraw and reflect their true location  
	 */
	public void updateMap(){
		if(shouldDraw == true && map != null && getActivity() != null){
			IconGenerator iconGenGreen = new IconGenerator(getActivity());
			iconGenGreen.setStyle(IconGenerator.STYLE_GREEN);
			IconGenerator iconGenOrange = new IconGenerator(getActivity());
			iconGenOrange.setStyle(IconGenerator.STYLE_ORANGE);
			IconGenerator iconGenRed = new IconGenerator(getActivity());
			iconGenRed.setStyle(IconGenerator.STYLE_RED);
			IconGenerator iconGenBlack = new IconGenerator(getActivity());
			iconGenBlack.setStyle(IconGenerator.STYLE_BLACK);		
			
			map.clear();
			if(zoneList != null){
				for (Zone zone : zoneList) {
					MarkerOptions marker = zone.getMarker();
					if(marker == null){
						setUpZoneMarkers(); //this fix the issue with slow network  
					}
					if(marker != null){
						if(zone.isNormalZone()){
							if(zone.getPanicLevel() <= 20){
								marker.icon(BitmapDescriptorFactory.fromBitmap(iconGenGreen.makeIcon(zone.getName())));
							}
							else if(zone.getPanicLevel() <= 60){
								marker.icon(BitmapDescriptorFactory.fromBitmap(iconGenOrange.makeIcon(zone.getName())));
							}
							else if(zone.getPanicLevel() <= 90){
								marker.icon(BitmapDescriptorFactory.fromBitmap(iconGenRed.makeIcon(zone.getName())));
							}
							else{
								marker.icon(BitmapDescriptorFactory.fromBitmap(iconGenBlack.makeIcon(zone.getName())));
							}	
						}
						map.addMarker(marker);
					}
				}
			}
			
			IconGenerator iconGen2 = new IconGenerator(getActivity());
			iconGen2.setStyle(IconGenerator.STYLE_BLUE);
			for(PlayerMarker pl : AllPlayerMarkers.getInstance().getMarkerList()){
				MarkerOptions mark = new MarkerOptions();
				mark.position(new LatLng(pl.getLatitude(), pl.getLongitude()));
				mark.icon(BitmapDescriptorFactory.fromBitmap(iconGen2.makeIcon(pl.getNickname())));
				map.addMarker(mark);
			}
		}
	}
	
	/**
	 * A simple getter that returns the zones within the map
	 * @return the list of the zones
	 */
	public ArrayList<Zone> getZoneList() {
		return zoneList;
	}

	/** 
	 * Stop the thread that keep track on when to update the markers
	 */
	public void closeApplication(){
		mapUpdateTimer.stop();
	}
	
	/**
	 * Function that should draw lines between zones.
	 */
	public void drawlines(){
		shouldDraw = false;
		Zone z1 = zoneList.get(0);
		Zone z2 = zoneList.get(1);
		
		
		 Polyline line = map.addPolyline(new PolylineOptions()
	     .add(new LatLng(z1.getLatitude(), z1.getLongitude()), new LatLng(z2.getLatitude(), z2.getLongitude()))
	     .width(3)
	     .color(Color.BLACK));
		 		 
		 Polyline line2 = map.addPolyline(new PolylineOptions()
	     .add(new LatLng(z2.getLatitude()+0.0001, z2.getLongitude()+0.0001), new LatLng(z1.getLatitude()+0.0001, z1.getLongitude()+0.0001))
	     .width(3)
	     .color(Color.BLACK));
		 
		 
	}
	
	/**
	 * Will set the boolean which tells if the map should be re-drawn to false.
	 * This happens at the end during the reflection process. 
	 */
	public void stopDrawing(){
		shouldDraw = false;
	}
	
}
