package dont.panic.renewed.fragments;

import java.text.BreakIterator;
import java.util.ArrayList;

import dont.panic.renewed.R;
import dont.panic.renewed.interfaces.ActionPerformedListenerInterface;
import dont.panic.renewed.interfaces.LocationlistenerInterface;
import dont.panic.renewed.interfaces.ZoneListener;
import dont.panic.renewed.models.Civilian;
import dont.panic.renewed.models.Player;
import dont.panic.renewed.models.Zone;
import dont.panic.renewed.models.blockade.Blockade;
import dont.panic.renewed.models.equipment.Equipment;
import dont.panic.renewed.models.equipment.carryable.civiliantype.Medkit;
import dont.panic.renewed.models.equipment.carryable.civiliantype.Megaphone;
import dont.panic.renewed.utils.ActionNamesAndValue;
import dont.panic.renewed.utils.EquipAndBlockNames;
import dont.panic.renewed.utils.EquipmentTimeSetting;
import dont.panic.renewed.utils.EquipmentType;
import dont.panic.renewed.utils.GlobalZoneStatusHelper;
import dont.panic.renewed.utils.RolesNamesAndBenefis;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ExpandableListView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.view.View.OnClickListener;

/**
 * ZomeFragment is the fragment that displays the option once the player is inside a "valid/event" gps location.
 * Action such as break down the door, or interact with the user.
 */

public class ZoneFragment extends Fragment implements ZoneListener, OnClickListener{

	
	private ArrayAdapter<String> civilianListViewAdapter;
	private View rootView;
	private LayoutInflater inflater;
	private ViewGroup container;
	private int progress;	
	private ListView civiliansListView;
	private ArrayList<String> civList;
	private ActionPerformedListenerInterface observer;
	
	/**
	 * The typical fragment lifespan call that is used to initialize the layout and such.
	 * 
	 * @see android.support.v4.app.Fragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		// TODO Auto-generated method stub
		rootView = inflater.inflate(R.layout.fragment_zone_fragment,container,false );
		this.inflater = inflater;
		this.container = container;
		
		GlobalZoneStatusHelper.getInstance().addListener(this);
		
		Button equipButton = (Button) rootView.findViewById(R.id.fragment_zone_fragment_use_equipment_button);
		equipButton.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				if(Player.getInstance().isCarryingEquipmentOfType(EquipmentType.CIVILIAN)){
					setUpEquipmentCivilianActionDialoge();
				}
				else{
					new AlertDialog.Builder(getActivity()).setMessage(("Sorry, you have no items")).show();
				}	
			}
		});
		
		return rootView;
	}
	
	/**
	 * Set up the listener that is interested in knowing when an action has been done.
	 * @param ob The object that wants to be notified when an action is performed in the zone GUI.
	 */
	public void setActionPerformedObserver(ActionPerformedListenerInterface ob){
		this.observer = ob; 
	}
	
	
	/**
	 * The function that sets up the GUI element representing all the civilians in a standard android ListView
	 */
	private void setUpCivlianListGUI(){

		if(getActivity() != null){ //not sure why, but it seems like if the user restart the app to fast, this fragment which belongs to the old activity is still 
								   //in memory somehow. This if statement prevent that it tries to communicate with the none-existing closed activity	
		updateCiviliansStatus();
		civiliansListView = (ListView) inflater.inflate(R.layout.reuseable_listview, container,false);
		civilianListViewAdapter = new ArrayAdapter<String>(getActivity(), android.R.layout.simple_list_item_1, civList);
		civiliansListView.setAdapter(civilianListViewAdapter);
	
		LinearLayout lin =(LinearLayout) rootView.findViewById(R.id.fragment_zone_fragment_npc_container);
		lin.removeAllViews();

		lin.addView(civiliansListView);
		civiliansListView.setOnItemClickListener( new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int position,
					long id) {
				setUpCivilianActionDialoge(GlobalZoneStatusHelper.getInstance().getCurrentZone().getCivilianList().get(position));
				
			}
		});
		
		}
	}

	
	/**
	 * The function will be called when the CustonMapFragment notify that the player has entered or is in a zone.
	 * This sets up the GUI for the zone, depending on if its a zone, a secure camp and if the zone is locked/blocked or not. 
	 * @see dont.panic.renewed.interfaces.ZoneListener#onZoneEnter()
	 */
	
	@Override
	public void onZoneEnter() {
		Zone zone = GlobalZoneStatusHelper.getInstance().getCurrentZone();
		if(zone != null){
			if(zone.isNormalZone()){
				if(zone.isLocked()){
					LinearLayout lin =(LinearLayout) rootView.findViewById(R.id.fragment_zone_fragment_blockade_container);
					for (Blockade blockade : zone.getBlockadeList()) {
						if(blockade.isLocked()){
							View view = inflater.inflate(R.layout.reuseable_blockade_layout, container, false);
							TextView text= (TextView) view.findViewById(R.id.reuseable_blockade_layout_textview_name);
							Button btn = (Button) view.findViewById(R.id.reuseable_blockade_layout_button_action_perform);
							text.setText("You must remove the blockade before you can enter!");
							btn.setText(blockade.getName());
							btn.setTag(blockade.getName());
							btn.setOnClickListener(this);
							lin.addView(view);
						}
					}
				}
				else{
					setUpCivlianListGUI();
					LinearLayout lin = (LinearLayout) rootView.findViewById(R.id.fragment_zone_fragment_blockade_container);
					lin.removeAllViews();
					Button eq = (Button) rootView.findViewById(R.id.fragment_zone_fragment_use_equipment_button);
					eq.setVisibility(eq.VISIBLE);
				}
			}
			else{
				onSaftyZoneEnter();
			}
		}
		
	}

	/**
	 * This function is called when the CustomMapFragment detect that the player has left the zone. It will clear the zone GUI so that its
	 * blank/empty
	 * @see dont.panic.renewed.interfaces.ZoneListener#onZoneExit()
	 */
	@Override
	public void onZoneExit() {
		LinearLayout lin = (LinearLayout) this.rootView.findViewById(R.id.fragment_zone_fragment_blockade_container);
		lin.removeAllViews();
		
		LinearLayout lin2 = (LinearLayout) this.rootView.findViewById(R.id.fragment_zone_fragment_npc_container);
		lin2.removeAllViews();
		
		LinearLayout lin3 = (LinearLayout) this.rootView.findViewById(R.id.fragment_zone_fragment_equipment_container);
		lin3.removeAllViews();
		Button eq = (Button) rootView.findViewById(R.id.fragment_zone_fragment_use_equipment_button);
		eq.setVisibility(eq.INVISIBLE);
		
	}

	/**
	 * This function is called when the user clicks/selects a blockade.
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	@Override
	public void onClick(final View v) {
		Zone zone = GlobalZoneStatusHelper.getInstance().getCurrentZone();
		if(zone != null){
			for (final Blockade blockade : zone.getBlockadeList()) {
				if(blockade.getName().equals(v.getTag().toString())){
					final Button btn = (Button) v;		
					if(Player.getInstance().getRole().equalsIgnoreCase(RolesNamesAndBenefis.FIREFIGHTER)
							|| Player.getInstance().getRole().equalsIgnoreCase(RolesNamesAndBenefis.VOLUNTEER)){
					setUpBlockadeActionDialoge(v, blockade, btn);
					}
					else{
						setUpWrongRoleDialog("Sorry!", "Your role cannot remove blockades!");
					}
				}
			}			
		}
		
	}


	/**
	 * This function sets up a generic dialog window, we use it to set up "error" messages such as wrong role, or if the player
	 * tried to heal/calm a dead civilian. 
	 * @param title Title of the dialog window
	 * @param body The content of the message
	 */
	private void setUpWrongRoleDialog(String title, String body){
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle(title);
		
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {						
			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub							
			}
		});
		
		builder.setMessage(body);
		builder.show().setCanceledOnTouchOutside(true);
	}
	
	/**
	 * Sets up a dialog window asking the user what item he want to use to remove the blockade
	 * @param v Alternative the view that represent the blockade (not in use, but will allow us to use images to represent the blockade)
	 * @param blockade The blockade the player pressed on
	 * @param btn The button object representing the blockade 
	 */
	
	private void setUpBlockadeActionDialoge(final View v, final Blockade blockade,
			final Button btn) {
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("What do you want to do?");
		
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {						
			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub							
			}
		});
		
		final ArrayList<Equipment> eqList = new ArrayList<Equipment>();
		for(Equipment equipment:Player.getInstance().getEquipmentList()){
			if(EquipAndBlockNames.getIfEquipmentIsUsableOnBlockade(btn.getText().toString(), equipment.getName()))
				eqList.add(equipment);
		}
		
		for(Equipment equipment:GlobalZoneStatusHelper.getInstance().getCurrentZone().getEquipmentInZoneList()){
			if(EquipAndBlockNames.getIfEquipmentIsUsableOnBlockade(btn.getText().toString(), equipment.getName()))
				eqList.add(equipment);
		}
		
		
		if(eqList.size() > 0){
			final String[] eqArray= new String[eqList.size()];
			for(int i =0; i < eqList.size(); i++){
				eqArray[i] = eqList.get(i).getName()+"   ("+(eqList.get(i).getTimeRequired()+RolesNamesAndBenefis.getBlockadeRemovalTimePenalty(Player.getInstance().getRole()))+"s)"; 
			}
			
			builder.setItems(eqArray, new DialogInterface.OnClickListener() {						
				@Override
				public void onClick(DialogInterface dialog, int which) {
					startProgressBar(eqList.get(which).getName(),eqList.get(which).getTimeRequired()
							+RolesNamesAndBenefis.getBlockadeRemovalTimePenalty(Player.getInstance().getRole()),"block", blockade.getName(),"");
					blockade.unlock();
					observer.onZoneStartUnlocking(eqList.get(which).getName());
	
				}
			});
			
			AlertDialog alert = builder.create();
			alert.setCanceledOnTouchOutside(true);
			alert.show();
		}
		else{
			setUpWrongRoleDialog("Sorry!", "You have no useable items");
		}
			
	}
	/**
	 * Sets up the action dialog for civilians. This is triggered when the player presses the object representing a civilian
	 * @param civilian The civilians that was pushed/pressed
	 */
	private void setUpCivilianActionDialoge(final Civilian civilian) {
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("What do you want to do?");
		
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {						
			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub							
			}
		});
		String[] options;
		if(!Player.getInstance().getRole().equalsIgnoreCase(RolesNamesAndBenefis.FIREFIGHTER)){
		options = new String[2];
		options[0] = "Reduce panic with 5 (" +(EquipmentTimeSetting.TIME_REQUIRED_ON_CALMING+RolesNamesAndBenefis.getReducePanicTimePenalty(Player.getInstance().getRole())) +"s)" ;
		options[1] = "Restore health with 5 (" +(EquipmentTimeSetting.TIME_REQUIRED_ON_HEALING +RolesNamesAndBenefis.getHealingTimePenalty(Player.getInstance().getRole())) +"s)";
		
		}
		else{
			options = new String[1];
			options[0] = "Reduce panic with 5 (" +(EquipmentTimeSetting.TIME_REQUIRED_ON_CALMING+RolesNamesAndBenefis.getReducePanicTimePenalty(Player.getInstance().getRole())) +"s)";
		}
		
		builder.setItems(options, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				if(civilian.isAlive()){
					switch (which) {
					case 0:
//						civilian.applyAction(ActionNamesAndValue.CALMING);
//						updateCiviliansStatus();
						startProgressBar(ActionNamesAndValue.CALMING,EquipmentTimeSetting.TIME_REQUIRED_ON_CALMING+RolesNamesAndBenefis.getReducePanicTimePenalty(Player.getInstance().getRole()),
								"action",ActionNamesAndValue.CALMING, civilian.getName());
						observer.onCivilianActionStartUsing(GlobalZoneStatusHelper.getInstance().getCurrentZone().getName(), civilian.getName(), ActionNamesAndValue.CALMING);
						break;
					case 1:
//						civilian.applyAction(ActionNamesAndValue.HEALING);
//						updateCiviliansStatus();
						startProgressBar(ActionNamesAndValue.HEALING,EquipmentTimeSetting.TIME_REQUIRED_ON_HEALING +RolesNamesAndBenefis.getHealingTimePenalty(Player.getInstance().getRole()),
								"action",ActionNamesAndValue.HEALING, civilian.getName());
						observer.onCivilianActionStartUsing(GlobalZoneStatusHelper.getInstance().getCurrentZone().getName(), civilian.getName(), ActionNamesAndValue.HEALING);
						break;
	
					default:
						break;
					}
				}
				else{
					new AlertDialog.Builder(getActivity()).setMessage("Sorry the civilian is dead....").show();
				}
			}
		});
		
		
		AlertDialog alert = builder.create();
		alert.setCanceledOnTouchOutside(true);
		alert.show();
		
	}
	
	/**
	 * Sets up the action dialog for equipment usage directed toward the civilian such as the Medkit and Megaphone.
	 */
	private void setUpEquipmentCivilianActionDialoge() {
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("What do you want to do?");
		
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {						
			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub							
			}
		});
		
		final String[] options = Player.getInstance().getEquipmentOfType(EquipmentType.CIVILIAN, RolesNamesAndBenefis.getItemUsageTimePenalty(Player.getInstance().getRole()));
		builder.setItems(options, new DialogInterface.OnClickListener() {			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				String eqName = options[which].split("\n")[0];
				String beak ="t";
				if(eqName.equalsIgnoreCase(EquipAndBlockNames.MEDICAL_KIT)){
					Medkit kit = new Medkit();
					startProgressBar(eqName, (kit.getTimeRequired()+RolesNamesAndBenefis.getItemUsageTimePenalty(Player.getInstance().getRole())), "civAll",eqName, "");
					observer.onCivilianItemStartUsing(GlobalZoneStatusHelper.getInstance().getCurrentZone().getName(), eqName);

				}
				else if(eqName.equalsIgnoreCase(EquipAndBlockNames.MEGAPHONE)){
					Megaphone mega = new Megaphone();
					startProgressBar(eqName, (mega.getTimeRequired()+RolesNamesAndBenefis.getItemUsageTimePenalty(Player.getInstance().getRole())), "civAll",eqName,"");
					observer.onCivilianItemStartUsing(GlobalZoneStatusHelper.getInstance().getCurrentZone().getName(), eqName);

				}
				
				
			}
		});
		AlertDialog alert = builder.create();
		alert.setCanceledOnTouchOutside(true);
		alert.show();
	}
	
	/**
	 * The function creates a generic action window with a progressbar. The action window is customized by using the 
	 * input parameter 
	 * @param itemName The action/equipment that was done/used
	 * @param max The max amount of the progress bar
	 * @param action  The type of the action (used for sending to the server)
	 * @param causeName The effect of the equipment (used for sending the effect to the server)
	 * @param civName The name of the civilian the action is performed on
	 */
	
	private void startProgressBar(final String itemName, int max, final String action, final String causeName, final String civName){
		final ProgressDialog progressBar = new ProgressDialog(this.getActivity());
		progressBar.setCancelable(false);
		progressBar.setMessage("Using " +itemName);
		progressBar.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		progressBar.setProgress(0);
		progressBar.setMax(max);
		progressBar.show();
		progress = 0;
		
		final Handler progessBarHandler = new Handler();


		new Thread(new Runnable() {
			  public void run() {
				while (progress < progressBar.getMax()) {

				  // your computer is too fast, sleep 1 second
				  try {
					Thread.sleep(1000);
				  } catch (InterruptedException e) {
					e.printStackTrace();
				  }
				  progress++;

				  // Update the progress bar
				  progessBarHandler.post(new Runnable() {
					public void run() {
					  progressBar.setProgress(progress);
					}
				  });
				  
				  if(progress >= progressBar.getMax()){
					  progressBar.dismiss();
					  Player.getInstance().removeEquipment(itemName);
					  if(observer != null){
						  if(action.equalsIgnoreCase("block")){  
								  observer.onZoneUnlocked(GlobalZoneStatusHelper.getInstance().getCurrentZone().getName(), causeName);
								  observer.onEquipmentUsed(itemName);
								  
								  
								  getActivity().runOnUiThread(new Runnable() {
									    public void run() {
									    	Zone zone = GlobalZoneStatusHelper.getInstance().getCurrentZone();
											boolean locked = false;
											for(Blockade blockade : zone.getBlockadeList() ){
												if(blockade.isLocked() == true){
													locked = true;
													break;
												}					
											}
											
											if(locked == false){
												Button eq = (Button) rootView.findViewById(R.id.fragment_zone_fragment_use_equipment_button);
												eq.setVisibility(eq.VISIBLE);
												onZoneEnter();
											}
									    }
									});
									
								  
								  
						  }
						  if(action.equalsIgnoreCase("action")){
							  
							  observer.onCivilianAction(GlobalZoneStatusHelper.getInstance().getCurrentZone().getName(), civName, causeName);
							  for(final Civilian civ : GlobalZoneStatusHelper.getInstance().getCurrentZone().getCivilianList()){
								  if(civ.getName().equalsIgnoreCase(civName)){
									  if(causeName.equalsIgnoreCase(ActionNamesAndValue.CALMING)){
										  
										  
										  getActivity().runOnUiThread(new Runnable() {
											
											@Override
											public void run() {
												civ.applyAction(ActionNamesAndValue.CALMING);
												updateCiviliansStatus();
												
											}
										});
									  }
									  if(causeName.equalsIgnoreCase(ActionNamesAndValue.HEALING)){
										  getActivity().runOnUiThread(new Runnable() {
												
												@Override
												public void run() {
													civ.applyAction(ActionNamesAndValue.HEALING);
													updateCiviliansStatus();
													
												}
											});
									  }
									  
								  }
							  }
							  
							  
						  }
						  
						  if(action.equalsIgnoreCase("civAll")){
							  observer.onCivilianItemUsed(GlobalZoneStatusHelper.getInstance().getCurrentZone().getName(), causeName);
							  observer.onEquipmentUsed(itemName);
							  
							  for(Civilian c : GlobalZoneStatusHelper.getInstance().getCurrentZone().getCivilianList()){
								  c.applyEquipment(causeName);
							  }
							  
							  getActivity().runOnUiThread(new Runnable() {
								
								@Override
								public void run() {
									updateCiviliansStatus();
									
								}
							});
							  
						  }
					  } 
				  }
				}
			  }
				
	       }).start();
	}


	/**
	 * This function is called when a new game tick is received. It update the status of the civiliance and refresh the
	 * GUI if the player is currently in a zone.   
	 */
	public void updateCiviliansStatus(){
		if(civList ==  null)
			civList = new ArrayList<String>();
		
		civList.clear();
		if(GlobalZoneStatusHelper.getInstance().getCurrentZone().getCivilianList() != null){
			for(Civilian civ : GlobalZoneStatusHelper.getInstance().getCurrentZone().getCivilianList()){
				civList.add(civ.getName() + "  health:"+ civ.getHealth() + "  panic:" +civ.getPanic());
			}		
			if(civilianListViewAdapter != null)
				civilianListViewAdapter.notifyDataSetChanged();
		}
	}
	
	/**
	 * Setting up a listview representing items the play can pick up  
	 */
	private void onSaftyZoneEnter(){
		if(getActivity() != null){
			ListView eqList = (ListView) inflater.inflate(R.layout.reuseable_listview, container,false);
			final ArrayList<String> eqNameList = EquipAndBlockNames.getAllEquipmentNames();
			eqList.setAdapter( new ArrayAdapter<String>(this.getActivity(), android.R.layout.simple_list_item_1, eqNameList));
			
			LinearLayout lin = (LinearLayout) rootView.findViewById(R.id.fragment_zone_fragment_equipment_container);
			lin.addView(eqList);
			
			eqList.setOnItemClickListener(new OnItemClickListener() {
	
				@Override
				public void onItemClick(AdapterView<?> arg0, View arg1, int pos,
						long id) {
					setUpEquipmentPickupDialoge(EquipAndBlockNames.getEquipmentByName(eqNameList.get(pos)));
					
				}
			});
		}
		
	}
	
	/**
	 * Create the action dialog representing the player backpack. When selecting an equipment to pick up
	 * the player must decide where to place it.
	 * @param eq The equipment the player wish to pick up
	 */
	private void setUpEquipmentPickupDialoge(final Equipment eq) {
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
		builder.setTitle("Where do you want to place it?");
		
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {						
			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub							
			}
		});
		
		String[] eqList = new String[Player.getInstance().getMaxItem()];
		for(int i = 0; i < Player.getInstance().getMaxItem(); i++ ){
			eqList[i] = "";
		}
		
		ArrayList<Equipment> eqArrayList = Player.getInstance().getEquipmentList();
		
		for(int i = 0; i < eqArrayList.size(); i++){
			eqList[i] = eqArrayList.get(i).getName();
		}
		
		builder.setItems(eqList, new DialogInterface.OnClickListener() {			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				
				if(Player.getInstance().getEquipmentList().size() < Player.getInstance().getMaxItem()){
					Player.getInstance().addEquipment(eq);
					observer.onEquipmentPickup(eq.getName());
				}
				else{
					observer.onEquipmentUsed(Player.getInstance().getEquipmentList().get(which).getName());
					Player.getInstance().removeEquipment(which);
					
					Player.getInstance().getEquipmentList().add(which, eq);
					observer.onEquipmentPickup(eq.getName());
				}
				
				
			}
		});
		
		AlertDialog alert = builder.create();
		alert.setCanceledOnTouchOutside(true);
		alert.show();
		
		
	}
	
	/**
	 * This function is called when the game session is over and the reflection answers are submitted and confirme by the server.
	 * This will replace the content of the zoneFragment with zone history.
	 */
	public void showZoneHistory(){
		LinearLayout block = (LinearLayout) rootView.findViewById(R.id.fragment_zone_fragment_blockade_container);
		block.removeAllViews();
		LinearLayout civ =(LinearLayout) rootView.findViewById(R.id.fragment_zone_fragment_npc_container);
		civ.removeAllViews();
		
		ListView zoneHistoryListView = (ListView) inflater.inflate(R.layout.reuseable_listview, container,false);
		
		ArrayList<String> test = Player.getInstance().getZoneHistory();
		zoneHistoryListView.setAdapter( new ArrayAdapter<String>(this.getActivity(), android.R.layout.simple_list_item_1, Player.getInstance().getZoneHistory()));
		LinearLayout lin = (LinearLayout) rootView.findViewById(R.id.fragment_zone_fragment_equipment_container);
		lin.removeAllViews();
		lin.addView(zoneHistoryListView);
	}
	
}
