package dont.panic.renewed.models;

import dont.panic.renewed.models.equipment.Equipment;
import dont.panic.renewed.models.equipment.carryable.civiliantype.EquipmentForCivilians;
import dont.panic.renewed.utils.ActionNamesAndValue;
import dont.panic.renewed.utils.EquipAndBlockNames;
import dont.panic.renewed.utils.EquipmentAndActionEffectAmountHelper;
import dont.panic.renewed.utils.EquipmentForCiviliansEffect;

/**
 * The object representing a civilian 
 *
 */
public class Civilian {

	/**
	 * The name of the civilian which will be displayed at the zone GUI
	 */
	private String name;
	
	/**
	 * The amount of health he has (0 = dead, 100 = max)
	 */
	private int health;
	
	
	/**
	 * The amount of panic he has ( 0 = min/good, 100 = max/bad)
	 */
	private int panic;
	
	/**
	 * A boolean telling if hes alive or not. This is used to check if the object has to be updated when a new game tick occures
	 */
	private boolean alive = true;
	
	public Civilian(String name) {
		this.name = name;
		health = 60;
		panic = 50;
	}

	/**
	 * A simple getter that return the civilians name
	 * @return the name of the civilian
	 */
	public String getName() {
		return name;
	}

	/**
	 * A simple setter which set the civilians name
	 * @param name the name the civilian will have
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 *  A simple getter that return the civilians health
	 * @return the civilian health
	 */
	public int getHealth() {
		return health;
	}

	/**
	 * A simple setter which set the civilians health
	 * @param health the health the civilian should have
	 */
	public void setHealth(int health) {
		this.health = health;
	}

	/**
	 *  A simple getter that return the civilians panic 
	 * @return the civilian panic
	 */
	public int getPanic() {
		return panic;
	}
	
	/**
	 * A simple setter which set the civilians panic
	 * @param panic the panic the civilian should have
	 */
	public void setPanic(int panic) {
		this.panic = panic;
	}
	/**
	 * This is called when a player use an action
	 * @param type the effect of the action
	 */
	public void applyAction(String type){
		if(type.equalsIgnoreCase(ActionNamesAndValue.CALMING)){
			int temp = panic - ActionNamesAndValue.getAmount(type);
			if(temp < 0)
				panic = 0;
			else
				panic = temp;
		}
		else if(type.equalsIgnoreCase(ActionNamesAndValue.HEALING)){
			int temp =  health + ActionNamesAndValue.getAmount(type);
			if(temp > 100)
				health = 100;
			else
				health = temp;
		}
	}
	
	/**
	 * This function is called when a player uses an item in the zone. It apply the benefits from the equipment upon the civiliance
	 * @param equipName
	 */
	public void applyEquipment(String equipName){
		EquipmentForCivilians eq =(EquipmentForCivilians) EquipAndBlockNames.getEquipmentByName(equipName);
		if(eq.getEffect().equals(EquipmentForCiviliansEffect.HEAL_ALL)){
			health = health + eq.getAmount();
			if(health > 100)
				health = 100;
				
		}
		else if(eq.getEffect().equals(EquipmentForCiviliansEffect.REDUCE_PANIC_ALL)){
			panic = panic - eq.getAmount();
			if(panic < 0)
				panic = 0;
		}
	}
	
	/**
	 * This is called when a panic wave occures.
	 */
	public void addPanic(int amount){
		this.panic = panic + amount;
		if(panic > 100)
			panic = 100;
	}
	
	/**
	 * This is called during a game tick.
	 * @param amount the amount of health to reduce
	 */
	public void reduceHealth(int amount){
		if(alive){
			if(panic >= 100){
				this.health = health - amount;
				if(health <= 0){
					health = 0;
					alive = false;
				}
			}
		}
	}

	/**
	 * 
	 * @return a boolean indicating if the civiliance is alive or not
	 */
	public boolean isAlive() {
		return alive;
	}
	
	
	
}
