package dont.panic.renewed.models;

import java.util.ArrayList;

import android.content.SharedPreferences;
import dont.panic.renewed.models.equipment.Equipment;
import dont.panic.renewed.models.equipment.carryable.blockadetype.Crowbar;
import dont.panic.renewed.models.equipment.carryable.blockadetype.FireExtinguisher;
import dont.panic.renewed.models.equipment.carryable.civiliantype.EquipmentForCivilians;
import dont.panic.renewed.models.equipment.carryable.civiliantype.Medkit;
import dont.panic.renewed.models.equipment.carryable.civiliantype.Megaphone;
import dont.panic.renewed.utils.EquipAndBlockNames;
import dont.panic.renewed.utils.EquipmentType;
import dont.panic.renewed.utils.RolesNamesAndBenefis;


/**
 * this class represent the player virtually
 */
public class Player {

	/**
	 * The name of the player which will be displayed to the other players
	 */
	private String nickname;
	
	/**
	 * The role of the civilians, note this will be set to predefine roles which are defined in the utility objects
	 */
	private String role;
	
	/**
	 * A list of equipment carried by the player.
	 */
	private ArrayList<Equipment> equipmentList;
	
	/**
	 * The instance of the player, the player uses the singleton pattern so that we make sure that there are only one player object existing in the game.
	 */
	private static Player instance;
	
	/**
	 * The variable telling how many items the player can hold at all time-
	 */
	private int maxItem = 3;
	
	private ReflectionData reflectionData;


	/**
	 * The initialising, note that this is were the player role is hardcoded. 
	 */
	public Player(){
		this.equipmentList = new ArrayList<Equipment>();
		this.role = RolesNamesAndBenefis.VOLUNTEER;
	}
	
	/**
	 * The only visible function to the outside world, it will return the instance of the player
	 * @return the instance of the player aka the only legal player object.
	 */
	public static Player getInstance(){
		if(instance == null){
			instance = new Player();
		}
		return instance;
	}

	/**
	 * A simple getter which will return the name of the player
	 * @return the name of the player
	 */
	public String getNickname() {
		return nickname;
	}

	/**
	 * A simple setter which set the nick of the player.
	 * @param nickname
	 */
	public void setNickname(String nickname) {
		this.nickname = nickname;
	}

	/**
	 * A simple getter which will return the role of the player
	 * @return the role of the player.
	 */
	public String getRole() {
		return role;
	}

	/**
	 * A simple setter which set the role of the player.
	 * @param role the role that the player should have
	 */
	public void setRole(String role) {
		this.role = role;
	}
	
	/**
	 * Remove an equipment from the player, this is done when he uses the equipment ingame
	 * @param name the name of equipment that was used.
	 */
	public void removeEquipment(String name){
		for(Equipment eq: equipmentList){
			if(eq.getName().equalsIgnoreCase(name)){
				equipmentList.remove(eq);
				break;
			}
		}
	}
	
	/**
	 * Remove an equipment from the player, this is done when he replace and item in the backpack with another.
	 * @param pos The position of the equipment.
	 */
	public void removeEquipment(int pos){
		equipmentList.remove(pos);
	}
	
	/**
	 * Add an equipment to the backpack, this is called when the player picks up and item ingame
	 * @param eq The equipment to be added.
	 */
	public void addEquipment(Equipment eq){
		if(equipmentList.size() < 3)
			equipmentList.add(eq);
		
	}
	/**
	 * A check to see if the player has an equipment of a specific type
	 * @param type the type of the equipment (blockade or civilian type)
	 * @return true if the player has a equipment of the specific type, else false.
	 */
	public boolean isCarryingEquipmentOfType(EquipmentType type){
		for(Equipment equipment : equipmentList){
			if(equipment.getEquipmentType().toString().equalsIgnoreCase(type.toString()) ){
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Loops trough the equipmentlist of the player and return equipment of a specific type (either civilian directed type or blockade directed type)
	 * @param type The type of the equipment (blockade or civilian type)
	 * @return A list will all the equipment of that type.
	 */
	public String[] getEquipmentOfType(EquipmentType type, int penalty){
		ArrayList<String> eqList = new ArrayList<String>();
			for(Equipment equipment : equipmentList){
				if(equipment.getEquipmentType().toString().equalsIgnoreCase(type.toString()) ){
					if(role.equalsIgnoreCase(RolesNamesAndBenefis.FIREFIGHTER) && equipment.getName().equalsIgnoreCase(EquipAndBlockNames.MEDICAL_KIT)){
						
					}
					else
						eqList.add(equipment.getDisplayText(penalty));
				}
			}

		return eqList.toArray(new String[eqList.size()]);
	}
	
	/**
	 * A function used for testing which allows us to add item to the backpack without having to do so manually in the game
	 */
	public void setUpTestEquipmentData(){
//		Equipment eq1 = new Crowbar();
//		equipmentList.add(eq1);
		Equipment eq2 = new Medkit();
		equipmentList.add(eq2);
		Equipment eq3 = new FireExtinguisher();	
		equipmentList.add(eq3);
		Equipment eq4 = new Megaphone();
		equipmentList.add(eq4);
		
	}
	
	
	
	public int getMaxItem() {
		return maxItem;
	}


	public void setMaxItem(int maxItem) {
		this.maxItem = maxItem;
	}
	
	public ArrayList<Equipment> getEquipmentList() {
		return equipmentList;
	}
	
	public void setReflectionData(ReflectionData data){
		this.reflectionData = data;
		ReflectionQuestions.getInstance().setAnswers(data.getReflectionAnswersGiven());
	}
	
	/**
	 * This function is called when the player reconnect to a started game session.
	 * The server will send a list of string with the name of the equipment which this function will go throught and create the 
	 * correct equipmentobject and add it to the equipmentlist.
	 * @param eqListFromServer
	 */
	public void recoverEquipment(ArrayList<String> eqListFromServer){
		equipmentList.clear();
		for(String name : eqListFromServer){
			Equipment eq = EquipAndBlockNames.getEquipmentByName(name);
			equipmentList.add(eq);
		}
		
	}
	
	public ArrayList<String> getZoneHistory(){
		return reflectionData.getZoneVisitedWithTimeStamp();
	}
	
	
	
}
