package dont.panic.renewed.network;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;

import dont.panic.renewed.interfaces.NetworkMessageInterface;
import dont.panic.renewed.models.Player;
import dont.panic.renewed.models.Zone;
import dont.panic.renewed.models.blockade.Blockade;

/**
 * This is the object that handle the communication with the server
 *
 */
public class GameClient extends Thread{
	private Socket clientSocket;
	private PrintStream os;
	private BufferedReader is;
	private ArrayList<NetworkMessageInterface> list;
	private String ip;
	private Boolean running;
	
	public GameClient(String ipAdress, NetworkMessageInterface ob){
		list = new ArrayList<NetworkMessageInterface>();
		list.add(ob);
		ip = ipAdress;

	}
	/**
	 * Start the thread which attempts to connect to the server.
	 * @see java.lang.Thread#run()
	 */
	
	public void run(){
		try {
			InetAddress ipAdress = InetAddress.getByName(ip);
			clientSocket = new Socket(ipAdress, 7474);
			clientSocket.setKeepAlive(true);
			os = new PrintStream(clientSocket.getOutputStream());
			is = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
			os.println("join|"+Player.getInstance().getNickname()+"|"+Player.getInstance().getRole());
			running = true;
			
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			list.get(0).onNetworkConectionFail("failed");
			e.printStackTrace();
			
			
		}	

		
		while(running){
			if(clientSocket != null){
				try {
					String line = is.readLine();
					list.get(0).onMessageRecieved(line);
				} catch (IOException e) {
					e.printStackTrace();
				}
				
			}
			else{
				list.get(0).onNetworkConnectionLost("connection lost");
			}
			
		}
	}
	
	/**
	 * Send a message to the server
	 * @param msg The text we wish to send 
	 */
	public void sendData(String msg){
		os.println(msg);

	}
	/**
	 * This is called when the player location is updated, this is so the server can transmit it to the team mates
	 * so that they can get the new location
	 * @param lat the new latitude
	 * @param lng the new longitude
	 */
	
	public void sendPlayerPositionMessage(String lat, String lng){
		String msg = "loc|"+Player.getInstance().getNickname()+"|"+lat+"~" +lng;
		sendData(msg);
	}
	
	/**
	 * This is called when the player performs an action on the blockade, atm its occures when the player has removed a blockade
	 * The server will distribute this to the other players so that their state also get updates.
	 * @param zoneName
	 * @param blockadeName
	 */
	public void sendPlayerZoneBlockadeActionDone(String zoneName, String blockadeName  ){
		String msg = "block|"+Player.getInstance().getNickname()+"|"+zoneName+"~"+blockadeName; 
		sendData(msg);
		
	}
	
	/**
	 * This is called when the player performs an action on the civilian, atm this is either healing or calming
	 * The server will distribute this to the other players so that their state also get updates.
	 * @param zoneName The name of the zone
	 * @param civilianName The name of the civilian
	 * @param actionType The action used
	 */
	public void sendPlayerZoneCivlianActionDone(String zoneName, String civilianName, String actionType){
		String msg = "civSingle|"+Player.getInstance().getNickname()+"|"+zoneName+"~"+civilianName+"~"+actionType;
		sendData(msg);
	}
	
	/**
	 * This is called when a player use an equipment in a zone directed to the civilians, atm its either the medkit or the megaphone
	  * The server will distribute this to the other players so that their state also get updates.
	 * @param zoneName
	 * @param itemName
	 */
	
	public void sendPlayerZoneCivlianItemActionDone(String zoneName, String itemName){
		String msg = "civAll|"+Player.getInstance().getNickname()+"|"+zoneName+"~"+itemName;
		sendData(msg);
	}
	
	/**
	 * This is used to update the server player state, so if we crash we can recover with items
	 */
	public void sendEquipmentPickUpAction(String itemName){
		String msg = "eqPickup|"+Player.getInstance().getNickname()+"|"+itemName;
		sendData(msg);
	}

	/**
	 * This is used to update the server player state, so if we crash we can recover with items
	 */
	
	public void sendEquipmentUsedAction(String itemName){
		String msg = "eqUsed|"+Player.getInstance().getNickname()+"|"+itemName;
		sendData(msg);
	}
	
	public void sendPlayerEmotion(String emotion){
		String linebreak = System.getProperty("line.separator");
		emotion = emotion.replace(linebreak, "");
		emotion = emotion.replace("|","");
		
		String msg = "emotion|"+Player.getInstance().getNickname()+"|"+emotion+"|"+"Not answered"; 
		sendData(msg);
	}
	
	public void sendZoneEntered(String zoneName){
		String msg = "zoneEntered|"+Player.getInstance().getNickname()+"|"+zoneName; 
		sendData(msg);
	}
	
	public void sendZoneLeft(){
		String msg = "zoneLeft|"+Player.getInstance().getNickname()+"|"+"space";
		sendData(msg);
	}
	
	public void sendPlayerZoneCivlianActionStarted(String zoneName, String civilianName, String actionType){
		String msg = "civSingleStarted|"+Player.getInstance().getNickname()+"|"+zoneName+"~"+civilianName+"~"+actionType;
		sendData(msg);
	}
	
	public void sendPlayerZoneCivlianItemActionStarted(String zoneName, String itemName){
		String msg = "civAllStarted|"+Player.getInstance().getNickname()+"|"+zoneName+"~"+itemName;
		sendData(msg);
	}
	
	public void sendPlayerZoneBlockadeActionStarted(String equipName){
		String msg = "blockStarted|"+Player.getInstance().getNickname()+"|"+equipName; 
		sendData(msg);
	}
	
	public void sendReflectionAnswers(String answers){
		String msg = "reflection|"+Player.getInstance().getNickname()+"|"+answers;
		sendData(msg);
	}
	
	
	public void closeConnection(){
		try {
			os.close();
			is.close();
			clientSocket.close();
			running = false;

			
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
}





