package dont.panic.renewed.utils;

import java.util.ArrayList;

import dont.panic.renewed.models.PlayerMarker;

/**
 * A utility class, used to keep track of team mates map markers
 *
 */
public class AllPlayerMarkers {

	/**
	 * The instance of this object (singleton pattern)
	 */
	private static AllPlayerMarkers instance;
	
	/**
	 * The list of all the team mates markers
	 */
	private ArrayList<PlayerMarker> markerList;
	
	
	/**
	 * Default constructor that just initialise the variables
	 */
	private AllPlayerMarkers(){
		markerList = new ArrayList<PlayerMarker>();
	}
	
	/**
	 * 
	 * @return the instance of this object
	 */
	public static AllPlayerMarkers getInstance(){
		if(instance == null)
			instance = new AllPlayerMarkers();
		return instance;
	}
	
	/**
	 * Add a new marker for a team player, this is done when the network message sends network message telling when a new player has joined, or that is allready joined
	 * @param nick
	 */
	
	public void addNewPlayermarker(String nick){
		boolean exist =false;
		for(PlayerMarker pm : markerList){
			if(pm.getNickname().equalsIgnoreCase(nick)){
				exist = true;
				break;
			}
		}
		if(exist == false){
			PlayerMarker playerMarker = new PlayerMarker(nick);
			markerList.add(playerMarker);
		}
	}
	
	/**
	 * This is called when the server sends a team mate player location update
	 * @param nick the nickname of the team mate
	 * @param lat the new latitude
	 * @param lng the new longitude
	 */
	public void updatePosition(String nick, Double lat, Double lng){
		for(PlayerMarker pm : markerList){
			if(pm.getNickname().equalsIgnoreCase(nick)){
				pm.setLatitude(lat);
				pm.setLongitude(lng);
			}
		}
	}

	/**
	 * 
	 * @return Returns the list of all the player markers
	 */
	public ArrayList<PlayerMarker> getMarkerList() {
		return markerList;
	}
}
