package dont.panic.renewed.utils;

import java.util.ArrayList;

import dont.panic.renewed.interfaces.ZoneListener;
import dont.panic.renewed.models.Zone;


/**
 * class used to pass information between diferendt object about the zone status
 *
 */
public class GlobalZoneStatusHelper {
	
	/**
	 * The instance of the object
	 */
	private static GlobalZoneStatusHelper instance;
	
	/**
	 * A boolean used to tell if the player is currently in a zone or not
	 */
	private boolean inZone;
	
	/**
	 * The currenty zone the player is in
	 */
	private Zone currentZone;
	
	/**
	 * A list of zone listener which want to know when the player leave or enter a zone
	 */
	private ArrayList<ZoneListener> zoneObservers;
	
	
	/**
	 * the typical constructor that initialize the variables.
	 */
	private GlobalZoneStatusHelper(){
		inZone = false;
		zoneObservers = new ArrayList<ZoneListener>();
	}
	
	/**
	 * 
	 * @return return the instance of the object
	 */
	public static GlobalZoneStatusHelper getInstance(){
		if(instance == null)
			instance = new GlobalZoneStatusHelper();
		
		return instance;
	}


	/**
	 * 	
	 * @return if the player is in a zone or not
	 */
	public boolean isInZone() {
		return inZone;
	}


	/**
	 * This is called when a player has left or entered a zone.
	 * @param inZone
	 */
	public void setInZone(boolean inZone) {
		if(inZone == true){
			for (ZoneListener observers : zoneObservers) {
				observers.onZoneEnter();
			}
		}
		else{
			for (ZoneListener observers : zoneObservers) {
				observers.onZoneExit();
			}
		}
			
		this.inZone = inZone;
		
	}


	/**
	 * @return return the zone that the player is currenty in
	 */
	public Zone getCurrentZone() {
		return currentZone;
	}


	/**
	 * Update the zone that the player has entered
	 * @param currentZone the zone that the player has entered
	 */
	public void setCurrentZone(Zone currentZone) {
		this.currentZone = currentZone;
	}
	
	/**
	 * Add listener that want to be notified when the player has entered or left a zone
	 * @param listener the object that want to be notified.
	 */
	public void addListener(ZoneListener listener){
		if(zoneObservers.contains(listener) == false){
			zoneObservers.add(listener);
		}
			
	}
	
	

}
