package client;

import interfaces.PlayerJoinInterface;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.InetAddress;
import java.net.Socket;

import models.Game;

import com.google.gson.Gson;

/**
 * This class represent a "player". This object holds the connection to a client at all time 
 *
 */
public class PlayerClientThread extends Thread {

	private BufferedReader is;
	private PrintStream os;
	private Socket clientSocket;
	private final PlayerClientThread[] threads; 
	private int maxClientCount;
	private String nick;
	private final Game game;
	private PlayerJoinInterface playerJoinObserver;
	
	public PlayerClientThread(Socket clientSocket, PlayerClientThread[] threads, int maxClient, Game game, PlayerJoinInterface observer){
		this.clientSocket = clientSocket;
		this.threads = threads;
		this.maxClientCount = maxClient;
		this.game = game;
		this.playerJoinObserver = observer;
		

	}

	/**
	 * Start this tread, it will constantly listen to the client
	 */
	public void run(){	     
	      try {
			is = new BufferedReader(new InputStreamReader(clientSocket.getInputStream(), "UTF-8"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	      try {
			os = new PrintStream(clientSocket.getOutputStream());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	    while(true){
	    	 os.println("msg|Enter nick");
	    	 try {
				String temp = is.readLine();
				String raw[] = temp.split("\\|");
				nick = raw[1];
				System.out.println("nick: " + nick);
				if(nick.length() > 0 && nick != null){
					 os.println("msg|Nickname accepted");
					 
					
					 if(game.playerExist(nick)){
					 	Gson gson = new Gson();
						 os.println("recoverEquipment|server|"+gson.toJson(game.getPlayerEquipmentList(nick)));
					 }
					 else{
//						 playerJoinObserver.onPlayerJoin(nick);
						 game.addPlayer(nick, raw[2]);
						 playerJoinObserver.onPlayerJoin(game.getPlayerByNick(nick));
					 }
					
					 for (int i = 0; i < maxClientCount; i++) {
			               if (threads[i] != null && threads[i].nick != null) {
			            	   os.println("join|"+threads[i].nick);
			            	   threads[i].os.println("join|"+nick);
			                 System.out.println("join|"+threads[i].nick);
			               }
			             }  
					break;
				}
			} catch (IOException e) {
			      synchronized (this) {
			          for (int i = 0; i < maxClientCount; i++) {
			            if (threads[i] == this) {
			              threads[i] = null;
			              try {
							is.close();
				              os.close();
				              clientSocket.close();
						} catch (IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}

			            }
			          }
			        }
				e.printStackTrace();
			}  	 
	   }
	    outer:
	     while(true){
	    	 String line;
			try {
				String temp = is.readLine();
				String raw[] = temp.split("\\|");
				String header = raw[0];
				String sender = raw[1];
				line = raw[2];
		    	 synchronized (this) {		    
		    		 
			    		 if(header.equalsIgnoreCase("loc")){
			    			 for (int i = 0; i < maxClientCount; i++) {
					               if (threads[i] != null && threads[i].nick != null && !threads[i].nick.equalsIgnoreCase(sender) ) {
					                 threads[i].os.println(temp);
//					                 System.out.println(temp);
					               }
					             }  
			    		 }
			    		 
			    		 else if(header.equalsIgnoreCase("block")){
			    			 if(raw[2] != null){
			    				 String[] content = raw[2].split("~");
			    				 game.unlockBlockadeInZone(content[0], content[1]);
			    				 game.showPlayerActionUsed(sender, "Blockade removed");
			    			 }
			    			 for (int i = 0; i < maxClientCount; i++) {
					               if (threads[i] != null && threads[i].nick != null && !threads[i].nick.equalsIgnoreCase(sender) ) {
					                 threads[i].os.println(temp);
					                 System.out.println(temp);
					               }
					             }  
			    		 }
			    		 else if(header.equalsIgnoreCase("civSingle")){
			    			 for (int i = 0; i < maxClientCount; i++) {
					               if (threads[i] != null && threads[i].nick != null && !threads[i].nick.equalsIgnoreCase(sender) ) {
					                 threads[i].os.println(temp);
					                	
					               }
	
					             }  
					       		String rawCivAction = raw[2];
			    				String civAction[] = rawCivAction.split("~");
			    				game.applyActionOnCivilian(civAction[0], civAction[1], civAction[2]);
			    				game.showActionUsageFinished(sender, civAction[1], civAction[2]);
			    				 System.out.println(temp);	
			    		 }
			    		 else if(header.equalsIgnoreCase("civAll")){
			    			 for (int i = 0; i < maxClientCount; i++) {
					               if (threads[i] != null && threads[i].nick != null && !threads[i].nick.equalsIgnoreCase(sender) ) {
					                 threads[i].os.println(temp);
					                 System.out.println(temp);
					               }
					             }
			 				String rawCivAction = raw[2];
							String civAction[] = rawCivAction.split("~");
							game.applyEquipmentOnCivilian(civAction[0],civAction[1]);
							game.showItemUsageFinished(sender, civAction[1]);
			    		 }
			    		 else if(header.equalsIgnoreCase("eqPickup")){
			    			 	game.addEquipmentFromPlayerEquipmentList(sender, raw[2]);
			    			 	System.out.println(sender+" picked up " + raw[2]);
			    		 }
			    		 else if(header.equalsIgnoreCase("eqUsed")){
			    			    game.removeEquipmentFromPlayerEquipmentList(sender, raw[2]);
			    			    System.out.println(sender+" dropped" + raw[2]);
			    		 }
			    		 else if(header.equalsIgnoreCase("zoneEntered")){
			    			 System.out.println("entered zone :" +sender+raw[2]);
			    			   game.playerEnterZone(sender, raw[2]);
			    		 }
			    		 else if(header.equalsIgnoreCase("zoneLeft")){
			    			   System.out.println("forlot zonen "+ sender+raw[2]);
			    			   game.playerLeaveZone(sender);
			    		 }
			    		 else if(header.equalsIgnoreCase("civSingleStarted")){
					       		String rawCivAction = raw[2];
			    				String civAction[] = rawCivAction.split("~");
//			    				game.applyActionOnCivilian(civAction[0], civAction[1], civAction[2]);
			    				game.showPlayerUsingActionOnCivilian(sender, civAction[2], civAction[1]);
			    		 }
			    		 else if(header.equalsIgnoreCase("civAllStarted")){
				 				String rawCivAction = raw[2];
								String civAction[] = rawCivAction.split("~");
								game.showPlayerUsingEquipmentOnCivilian(sender, civAction[1]);
			    		 }
			    		 else if(header.equalsIgnoreCase("blockStarted")){
			    			 if(raw[2] != null){
			    				 game.showPlayerRemovingBlockade(sender, raw[2]);
			    			 }
			    		 }
			    		 else if(header.equalsIgnoreCase("emotion")){
			    			 if(raw[2] != null){
			    				 game.addEmotionAnswerOnAPlayer(sender, raw[2]);
			    			 }
			    		 }
			    		 else if(header.equalsIgnoreCase("reflection")){
			    			 if(raw[2] != null){
			    				 game.setReflectionAnswerOnAPlayer(sender, raw[2]);
			    			 }
			    		 }
	    		 
		    	 }
			} catch (IOException |NullPointerException e) {
			      synchronized (this) {
			          for (int i = 0; i < maxClientCount; i++) {
			            if (threads[i] == this) {
			              threads[i] = null;
			              try {
							is.close();
				              os.close();
				              clientSocket.close();
				              break outer;
						} catch (IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}

			            }
			          }
			        }
				e.printStackTrace();
			}

	     }
  
	}
	
	/**
	 * Send a message to the client telling them that the game is paused, this is the meregency pause
	 */
	public synchronized void sendPauseMessage(){
		if(os != null){
			os.println("pause|pause");
		}
	}
	
	/**
	 * send a message to the client telling them the game is paused and to gather the emotion data from the player
	 * @param duration the duration of the pause
	 */
	public synchronized void sendNormalPauseMessage(int duration){
		if(os != null){
			os.println("normalPause|"+duration);
		}
	}
	
	/**
	 * send a message to the client, telling them the game session is done.
	 */
	public synchronized void sendgameFinishedMessage(){
		if(os != null){
			os.println("gameFinish|gameFinish");
		}
	}
	/* Test function which will trigger the application to draw lines between the zone history.
	public synchronized void sendDrawLinedMessage(){
		if(os != null){
			os.println("drawLines|drawLines");
		}
	}
	*/
	/**
	 * Send the reflection answers back to the application
	 * @param json the formated Json string which contain all the reflection answers.
	 */
	public synchronized void sendReflectionData(String json){
		if(os != null){
			os.println("reflectionData|"+json);
		}
	}
	
	
	/**
	 * a function allowing outside object to send msg to the client
	 * @param message the message we want to send to the client.
	 */
	
	public synchronized void sendMessage(String message){
		if(os != null){
			os.println(message);
		}
	}
	
	/**
	 * return the nick of the player
	 * @return the nick of the player the thread belongs too.
	 */
	public String getNick(){
		return nick;
	}
	
}
