package game;

import interfaces.LocationListenerInterace;
import interfaces.PlayerJoinInterface;
import interfaces.ReflectionRequestInterface;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.List;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;

import server.GameServer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import layout.PlayerLayout;
import layout.PlayerReviewLayout;
import models.Player;
import observers.ZoneStats;

/**
 * The main object which initialized the other objects.
 * The user interface is created in this class
 *
 */
public class Main extends JFrame implements PlayerJoinInterface, ReflectionRequestInterface, LocationListenerInterace {

	/**
	 * The object which act as a server, it handle the networking
	 */
	private GameServer gameServer;
	
	/**
	 * The timer which keep track on when to trigger a panicwave
	 */
	private Timer panicTimer;
	
	/**
	 * A count helper, which keep track on how long the panic wave has lasted.
	 */
	int panicCounter = 0;
	private JLabel panicLabel;
	
	/**
	 * display the time left for a game tick
	 */
	private JLabel gameTickLabel;
	
	/**
	 * display how long a game have lasted
	 */
	private JLabel gameDurationLabel;
	
	/**
	 * display how many civilians are still alive.
	 */
	private JLabel civilianAliveLabel;
	
	/**
	 * The timer which keep track on when the next game tick should occure.
	 */
	private Timer tickTimer;
	
	/**
	 * the count helper which tells how long until next game tick.
	 */
	int tickCounter = 0;
	
	/**
	 * The button which trigger a emergency pause
	 */
	private JButton pause;
	/**
	 * The button which start the timers
	 */
	private JButton start;
	
	/**
	 * The button which trigger the pause and emotion gather function. This should only be used to give an introduction to the players on how it looks like.
	 * The game will trigger this itself
	 */
	private JButton testPause;
	
	/**
	 * This button will trigger a switchover in the game application from "play"to "review" mode. This should only be used to give an introduction to
	 * the players on how it looks like. The game will trigger this automaticly at the right timing.
	 */
	private JButton done;
	
	/**
	 * The panel which contain all the GUI element at the top 
	 */
	private JPanel headPanel;
	/**
	 * The panel which contain the layout designed to show the plyers in game mode and review mode.
	 */
	private JPanel playerPanel;
	
	/**
	 * the panel which contain all the buttons.
	 */
	private JPanel bottomPanel;
	
	/**
	 * The timer which keep track on how long the game session has lasted
	 */
	private Timer gameDurationTimer;
	/**
	 * the count helper, which shows how long the game has lasted.
	 */
	private int gameDurationCounter = 1;
	
	/**
	 * The timer which keep track on when to send a new hearth ebat message.
	 */
	private Timer hearthBeatTimer;
	
	/**
	 * The container of the playerlayout which allows us the place more players then what a normal screen can normally be fitted.
	 */
	private JScrollPane playerScroll;
	/**
	 * The function that set up the user interface for the server
	 */
	public Main(){
		super("server");
		setUpManagerGUI();
		setUpHearthBeatTimer();
	}
	
	/**
	 * The function which will create the manager GUI
	 */
	public void setUpManagerGUI(){
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 450, 300);
		setLayout(new GridBagLayout());
		setVisible(true);
		GridBagConstraints con = new GridBagConstraints();
		con.gridx = 0;
		con.gridy = 0;
		
		headPanel = new JPanel(new FlowLayout(FlowLayout.LEADING, 20,20));
		setUpHeadPanel();
		this.add(headPanel, con);
		
		con.gridy = 1;
		playerPanel = new JPanel(new FlowLayout(FlowLayout.LEADING, 40,20));
//		playerPanel.setPreferredSize(new Dimension(1600, 600));

		playerScroll = new JScrollPane();
		playerScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		playerScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		playerScroll.setViewportView(playerPanel);
//		this.add(playerPanel,con);
		this.add(playerScroll,con);
		
		con.gridy = 2;
		bottomPanel = new JPanel(new FlowLayout());
		setUpBottomPanel();
		this.add(bottomPanel,con);

	}
	
	/**
	 * This function creats the GUI elements which are to be placed at the top of the managerGUI frame
	 */
	public void setUpHeadPanel(){
		panicLabel = new JLabel("Timer");
		JPanel panicPanel = new JPanel();
		panicPanel.add(panicLabel);
		headPanel.add(panicPanel);
		
		gameTickLabel = new JLabel("Game ticks");
		JPanel tickPanel = new JPanel();
		tickPanel.add(gameTickLabel);
		headPanel.add(tickPanel);
		
		gameDurationLabel = new JLabel("Game duration");
		headPanel.add(gameDurationLabel);
		
		civilianAliveLabel = new JLabel();
		JPanel civPanel = new JPanel();
		civPanel.add(civilianAliveLabel);
		headPanel.add(civPanel);
		

	}
	
	/**
	 * Create the gui elements which are to be placed at the bottom of the manager GUI frame
	 */
	public void setUpBottomPanel(){
		
		start = new JButton("Start");
		start.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				setUpPanicWaveTimer();
				setUpGameTickTimer();
				setUpGameDurationTimer();
				pause.setVisible(true);
				start.setVisible(false);
				bottomPanel.revalidate();
			}
		});
		
		
		
		pause = new JButton("Emergency pause");
		pause.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				emergencyPauseGame();
				civilianAliveLabel.setText("civ alive" +gameServer.getGame().getNumberOfCiviliansAlive()+"/"+ gameServer.getGame().getNumberOfCiviliansInGame());
			}
		});
		pause.setVisible(false);
		bottomPanel.add(start);
		bottomPanel.add(pause);
		
		testPause = new JButton("TestAutoPause");
		testPause.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				gameServer.pauseGameNormal();
			}
		});
		bottomPanel.add(testPause);
		
		done = new JButton("DoneTest");
		done.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				gameServer.gameFinish();
				changePlayerPanelToReview();
			}
		});
		
		bottomPanel.add(done);
		
		
		JButton testChangePlayerlayoutButton = new JButton("Force change mode");
		testChangePlayerlayoutButton.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				changePlayerPanelToReview();
			}
		});
		
		bottomPanel.add(testChangePlayerlayoutButton);
		
	}
	
	
	/**
	 * The function that sets up the panic timer. This function will run while the game is not paused, and trigger the panic waves
	 */
	
	public void setUpPanicWaveTimer(){
        ActionListener action = new ActionListener()
        {   
            @Override
            public void actionPerformed(ActionEvent event)
            {
            	
                if(panicCounter > gameServer.getGame().getPanicWaveInterval())
                {
                	panicTimer.stop();
                	panicTimer.start();
                    panicLabel.setText("Panicwave!!!!!!!!!!!!!!!");
                    gameServer.panicWave();
                    panicCounter = 0;
                }
                
                else
                {
                	int remain = gameServer.getGame().getPanicWaveInterval() - panicCounter;
                    panicLabel.setText("Next panicwave in: " + remain + " sec");
                    panicCounter++;
                }
            }
        };
        
        panicTimer = new Timer(1000,action);
        panicTimer.setInitialDelay(1000);
        panicTimer.start();
	}
	
	
	/**
	 * This function sets up the game tick timer. The timer will run while the game is not paused and will trigger the game ticks.
	 */
	
	public void setUpGameTickTimer(){
	       ActionListener action = new ActionListener()
	        {   
	            @Override
	            public void actionPerformed(ActionEvent event)
	            {
	            	
	                if(tickCounter > gameServer.getGame().getGameTickInterval())
	                {
	                	tickTimer.stop();
	                	tickTimer.start();
	                	gameServer.gameTick();
	                    tickCounter = 0;
	                    civilianAliveLabel.setText("civ alive" +gameServer.getGame().getNumberOfCiviliansAlive()+"/"+ gameServer.getGame().getNumberOfCiviliansInGame());
	                }
	                
	                else
	                {
	                	int remain = gameServer.getGame().getGameTickInterval() - tickCounter;
	                	gameTickLabel.setText("Next game tick in: " + remain + " sec");
	                	tickCounter++;
	                }
	            }
	        };
	        
	        tickTimer = new Timer(1000,action);
	        tickTimer.setInitialDelay(1000);
	        tickTimer.start();
	}
	
	/**
	 * This function will set up the game duration timer, which is in charge of triggering pauses and ending the game
	 */
	public void setUpGameDurationTimer(){
		 ActionListener action = new ActionListener()
	        {   
	            @Override
	            public void actionPerformed(ActionEvent event)
	            {
	            	//930
	                if(gameDurationCounter > 930)
	                {
	                	//call method which initialize pause message;
	                	gameDurationTimer.stop();
        				gameServer.gameFinish();
	    				changePlayerPanelToReview();
	                }
	                
	                else
	                {
	                    int minutes = gameDurationCounter/60;
	                    int seconds = (gameDurationCounter % 60);
	                    	gameDurationLabel.setText("Game duration: " +minutes + ":"+seconds);
	                	
	                    	//900
	                	if(gameDurationCounter % 900 == 0){
//		                	gameDurationTimer.stop();
//	        				gameServer.gameFinish();
	                		normalPauseGame();
	                		gameServer.getGame().setGameState(true);
	                		panicTimer.stop();
	                		tickTimer.stop();
	                	}
	                	//300
	                	else if(gameDurationCounter % 300 == 0){
	                		normalPauseGame();
	                	}
	                	gameDurationCounter++;
	                }
	            }
	        };
	        
	        gameDurationTimer = new Timer(1000,action);
	        gameDurationTimer.setInitialDelay(1000);
	        gameDurationTimer.start();
	}
	
	/**
	 * This function sets up the heathbeat timer which is in charge of triggering and sending the hearth beat message at regualare intervall 
	 */
	public void setUpHearthBeatTimer(){
		 ActionListener action = new ActionListener()
	        {   
	            @Override
	            public void actionPerformed(ActionEvent event)
	            {
	            	gameServer.hearthBeat();
	            }
	        
	        };
	        hearthBeatTimer = new Timer(2000, action);
	        hearthBeatTimer.start();
	}
	
	
	/**
	 * Start the server, which will accept the connection from the clients
	 */
	public synchronized void start(){
		gameServer = new GameServer(this);
		gameServer.start();
		
	}
	
	/**
	 * The function which will be automatically called when the project is compiled
	 * @param args does nothing
	 */
	public static void main(String[] args){
		Main main = new Main();
		main.start();
		
	}
	
	/**
	 * This function is called when the user press pause, it will prevent panic waves and game ticks from occures
	 */
	public void emergencyPauseGame(){
		if(panicTimer.isRunning() == true){
			panicTimer.stop();
			gameServer.pauseGame();
			tickTimer.stop();
			pause.setLabel("resume");
			gameDurationTimer.stop();
		}
		else{
			panicTimer.start();
			pause.setLabel("Emergency pause");
			tickTimer.start();
			gameDurationTimer.start();
		}
		
		
	}
	
	/**
	 * This function is called by the game duration timer, and will trigger the pause function in the application 
	 * that also function as the emotion gathering function.
	 */
	public void normalPauseGame(){
		panicTimer.stop();
		tickTimer.stop();
		gameDurationTimer.stop();
		gameServer.pauseGameNormal();
		
		 ActionListener action = new ActionListener()
	        {   
	            @Override
	            public void actionPerformed(ActionEvent event)
	            {
	            	Timer time = (Timer) event.getSource();
	            	time.stop();
	            	resumeGame();
	            }
	        };
		 
		 
		Timer timer = new Timer(gameServer.getGame().getPauseLenght(), action);
		timer.start();
	}
	
	/**
	 * This function will restart the timers
	 */
	public void resumeGame(){
		if(!gameServer.getGame().isGetDone()){
			panicTimer.start();
			tickTimer.start();
			
		}
		gameDurationTimer.start();

	}


	/**
	 * This function will create a player layout which shows the player action in realtime
	 */
	@Override
	public void onPlayerJoin(Player player) {
		if(!gameServer.getGame().isGetDone() && !player.getRole().equalsIgnoreCase("manager")){
			PlayerLayout pl = new PlayerLayout(player);
			playerPanel.add(pl);
			if(playerPanel.getComponentCount() < 4)
				revalidate();
			
			playerPanel.revalidate();
			playerScroll.revalidate();

			player.setZoneEnteredObserver(this);
		}
	}
	
	/**
	 * This function is called when the game session is done and will change the mode to "review" mode.
	 */
	public void changePlayerPanelToReview(){
		playerPanel.removeAll();
		
		for(Player player : gameServer.getGame().getPlayerList()){
			if(!player.getRole().equalsIgnoreCase("manager")){
				PlayerReviewLayout pr1 = new PlayerReviewLayout(this, player);
				playerPanel.add(pr1);
				playerPanel.revalidate();
			}
		}
	}


	/**
	 * This function will request reflection answer to a specific nick
	 */
	@Override
	public void requestReflectionQuestionsToBeAsked(String nick) {
		gameServer.reSendGameFinishToASinglePlayer(nick);
		
	}


	/**
	 * This function is called when a player enter a new zone, and the event will be added to the zone history list
	 */
	@Override
	public void onZoneEnter(String nick, String zoneName) {
		 int minutes = gameDurationCounter/60;
         int seconds = (gameDurationCounter % 60);
         String time = "" + minutes + ":"+ seconds+ "   ";
		gameServer.getGame().addZoneEnteredWithTimeStampToAPlayer(nick, time+zoneName);
		
	}


	/**
	 * This function will send a player the reflection answer which he previously submitted.
	 * This will allow the players to get the reflection data incase if the application crash or restart during the reflection process.
	 */
	@Override
	public void requestSendReflectionAndZoneDataToPlayer(String nick) {
		gameServer.sendReflectionDataToPlayer(nick);
		
	}
	
	
	
	
	
	
}
