package layout;

import interfaces.PlayerStatsInterface;
import interfaces.CivilianListenerInterface;

import javax.swing.AbstractListModel;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;

import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import models.Equipment;
import models.Player;
import observers.ZoneStats;

/**
 * The class that contains the GUI elements which are used to keep track of the active players in real time
 *
 */
public class PlayerLayout extends JPanel implements PlayerStatsInterface{

	/**
	 * The nickname of the player
	 */
	private JTextField playerNickText;
	
	/**
	 * The role of the player
	 */
	private JTextField playerRoleText;
	
	/**
	 * The action which the player are using or have used.
	 */
	private JTextField playerActionText;
	
	/**
	 * The equipment in backpack slot one
	 */
	private JTextField equipOneText;
	
	/**
	 * The equipment in backpack slot two
	 */
	private JTextField equipTwoText;
	
	/**
	 * The equipment in backpack slot three
	 */
	private JTextField equipThreeText;
	
	/**
	 * the custom model which are used to represent the zone and its content which the player are in.
	 */
	private ZoneStats zonestats;
	
	/**
	 * the name of the zone the player was previously inn
	 */
	private JTextField oldZoneText;
	
	/**
	 * The list which display the civilians in the zone
	 */
	private JList<String> civilianInZoneList;
	
	/**
	 * The list which display the equipment in the zone.
	 */
	private JList<String> equipmentInZoneList;
	
	/**
	 * the name of the zone the player is currently inn.
	 */
	private JTextField currentZoneText;
	
	/**
	 * Create the panel which will contain all the GUI elements.
	 * 
	 */
	public PlayerLayout(Player player) {
		
		player.setPlayerStatsObserver(this);
		zonestats = new ZoneStats();
		player.setZoneStatsObserver(zonestats);
		setBorder(new EmptyBorder(5, 5, 5, 5));
		setLayout(new BorderLayout(0, 0));
		setVisible(true);
		
		JPanel playerStatsPanel = new JPanel();
		playerStatsPanel.setLayout(new GridBagLayout());
		
		GridBagConstraints con = new GridBagConstraints();
		con.gridx = 0;
		con.gridy = 0;
		con.ipadx = 5;
		con.ipady = 5;
		playerNickText = new JTextField(player.getNick());
		
		playerStatsPanel.add(playerNickText, con);
		con.gridx = 1;
		con.gridy = 0;
		
		playerRoleText = new JTextField(player.getRole());
		
		playerStatsPanel.add(playerRoleText, con);
		
		con.insets = new Insets(35, 0, 0, 0);
		
		con.gridx = 0;
		con.gridy = 1;
		JLabel equipOne = new JLabel("Equipment 1:");
		playerStatsPanel.add(equipOne,con);
		
		
		
		equipOneText = new JTextField(15);
		con.gridx = 1;
		con.gridy = 1;	
		playerStatsPanel.add(equipOneText, con);
		
		con.insets = new Insets(0, 0, 0, 0);
		
		con.gridx = 0;
		con.gridy = 2;
		JLabel equipTwo = new JLabel("Equipment 2:");
		playerStatsPanel.add(equipTwo,con);
		
		equipTwoText = new JTextField(15);
		con.gridx = 1;
		con.gridy = 2;	
		playerStatsPanel.add(equipTwoText, con);
		
		
		con.gridx = 0;
		con.gridy = 3;
		JLabel equipThree = new JLabel("Equipment 2:");
		playerStatsPanel.add(equipThree,con);
		
		equipThreeText = new JTextField(15);
		con.gridx = 1;
		con.gridy = 3;	
		playerStatsPanel.add(equipThreeText, con);
		
		con.insets = new Insets(25, 0, 0, 0);
		
		
		con.gridx = 0;
		con.gridy = 4;
		JLabel oldZoneLabel = new JLabel("Last zone");
		playerStatsPanel.add(oldZoneLabel,con);
		
		
		
		con.gridx = 1;
		oldZoneText = new JTextField(15);
		playerStatsPanel.add(oldZoneText, con);
		
		con.insets = new Insets(0, 0, 0, 0);
		
		con.gridx = 0;
		con.gridy = 5;
		JLabel currentZoneLabel = new JLabel("Current in Zone..");
		playerStatsPanel.add(currentZoneLabel, con);
		
		con.gridx = 1;
		
		currentZoneText = new JTextField(15);
		playerStatsPanel.add(currentZoneText, con);
		
		con.insets = new Insets(25, 0, 0, 0);
		
		con.gridx = 0;
		con.gridy = 6;
		JLabel civInZone = new JLabel("Civ in zone");
		playerStatsPanel.add(civInZone,con);
		
		
		con.gridx = 1;
		JLabel eqInZone = new JLabel("Equip in zone");
		playerStatsPanel.add(eqInZone, con);
		con.insets = new Insets(0, 0, 0, 0);
		
		con.gridx = 0;
		con.gridy = 7;
		
		
		civilianInZoneList = new JList<String>();
		civilianInZoneList.setVisibleRowCount(6);
		civilianInZoneList.setPrototypeCellValue("Civ xxx hp: xx panic: xx");
		JScrollPane scrollCiv = new JScrollPane();
		
		scrollCiv.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollCiv.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollCiv.setViewportView(civilianInZoneList);
		civilianInZoneList.setModel(zonestats);
		playerStatsPanel.add(scrollCiv, con);
		
		con.gridx =1;
		
		DefaultListModel<String> equipmentInZoneListModel = new DefaultListModel<>();
		equipmentInZoneList = new JList(equipmentInZoneListModel);
		equipmentInZoneList.setVisibleRowCount(6);
		equipmentInZoneList.setPrototypeCellValue("fire hosexxx");
		JScrollPane scrollEq = new JScrollPane(equipmentInZoneList);
		
		playerStatsPanel.add(scrollEq,con);
		
//		playerStatsPanel.add(equipmentInZoneList, con);
		
		con.gridx = 0;
		con.gridy = 8;
		
		con.insets = new Insets(25, 0, 0, 0);
		
		

		JLabel actionLabel = new JLabel("Action:");
		playerStatsPanel.add(actionLabel,con);

		con.gridx = 1;
		playerActionText = new JTextField("",20);
		playerStatsPanel.add(playerActionText,con);
		
		
		this.add(playerStatsPanel);
		

	}



	/**
	 * This function is called when the player join the server and submitt his nick and role.
	 */
	@Override
	public void onRoleSet(String role) {
		playerRoleText.setText(role);
		
		
	}



	/**
	 * This function is called when the player is picking up an equipment. The GUI will update and display the new equipment picked up
	 */
	@Override
	public void onEquipmentPickup(String equipment) {
		if(equipOneText.getText().length() < 1){
			equipOneText.setText(equipment);
		}
		else if(equipTwoText.getText().length() < 1){
			equipTwoText.setText(equipment);
		}
		else if(equipThreeText.getText().length() < 1){
			equipThreeText.setText(equipment);
		}
		
	}



	/**
	 * This function is called when the player use an equipment. The GUI will update and remove the equipment used
	 */
	@Override
	public void onEquipmentUsed(String equipment) {
		if(equipThreeText.getText().equalsIgnoreCase(equipment)){
			equipThreeText.setText("");
		}
		else if(equipTwoText.getText().equalsIgnoreCase(equipment)){
			equipTwoText.setText("");
		}
		else if(equipOneText.getText().equalsIgnoreCase(equipment)){
			equipOneText.setText("");
		}
		
	}

	/**
	 * This function is called when a player started using an action or equipment, or when its completed
	 */
	@Override
	public void onActionUsed(String msg) {
		playerActionText.setText(msg);
		
	}

	/**
	 * This function is called when a player enter a new zone. This will update the GUI element to show the content of the new zone
	 */
	@Override
	public void onZoneEntered(String ZoneName) {
		currentZoneText.setText(ZoneName);
		DefaultListModel<String> listModel = (DefaultListModel<String>) equipmentInZoneList.getModel();
		listModel.clear();
		for(String eqName : zonestats.getCurrentZone().getEquipmentInZoneList()){
			listModel.addElement(eqName);
		}
		this.revalidate();
	}

	/**
	 * This function is called when a player leave a zone. This will update the GUI element to remove the content of the zone.
	 */
	@Override
	public void onZoneLeft(String ZoneName) {
		oldZoneText.setText(ZoneName);
		currentZoneText.setText("");
		DefaultListModel<String> listModel = (DefaultListModel<String>) equipmentInZoneList.getModel();
		listModel.clear();
	//	this.validate();
	}

}
