package layout;

import interfaces.ReflectionAnswersInterface;
import interfaces.ReflectionRequestInterface;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.border.LineBorder;

import models.Player;

/**
 * The GUI class that display the elements needed in the "review" mode of the game.
 *
 */
public class PlayerReviewLayout extends JPanel implements ReflectionAnswersInterface {
	
	/**
	 * The tab GUI element which allows us to place the reflection answers given and zone history at the same place with switchable tabs
	 */
	JTabbedPane statsTabPane;
	
	/**
	 * The panel which contain the zone history and is to be placed inside the tabpane
	 */
	JPanel zoneHistoryTab;
	
	/**
	 * the panel that contains the reflection answers given and is to be placed within the tabpane
	 */
	JPanel reflectionAnswerTab;
	
	/**
	 * the nick of the player
	 */
	JLabel nick;
	
	/**
	 * the role of the player
	 */
	JLabel role;
	
	/**
	 * The player object this GUI elements belong to.
	 */
	Player player;
	
	/**
	 * The list of equipments left.
	 */
	JList equipmentList;
	
	/**
	 * Display the emotion at the various stages
	 */
	JLabel em1,em2,em3; //used to display the emotion on various stages 
	
	/**
	 * Display the reflection answers given at the various stages.
	 */
	JLabel q1,q2,q3,q4,q5,q6,q7,q8,q9,q10,q11; //used to display the questions and the player answers for the reflection questions.
	
	/**
	 * The obsers which want to be notified if the backup solution are triggered (send reflection data to application and re-request refection data from user) 
	 */
	private ReflectionRequestInterface reflectionObserver;
	
	/**
	 * The button which will trigger the backup method that will re-request the reflection answer from the player
	 */
	JButton reSendGameDoneMessage;
	
	/**
	 * The button which will trigger the backup method that will send out the reflection data submitted by the player.
	 */
	JButton sendOutData;
	
	/**
	 * The panel that contains the buttons.
	 */
	JPanel buttonPanel;
	
	
	
	/**
	 * Sets up the GUI and register the observsrs.
	 * @param ob The object that will act as an observer
	 * @param pl the player model which this class is to represent
	 */
	public PlayerReviewLayout(ReflectionRequestInterface ob, Player pl){
		this.setLayout(new GridBagLayout());
		this.setBorder(new LineBorder(Color.BLACK));
		reflectionObserver = ob;
		player = pl;
		player.setReflectionAnswerObserver(this);
		
//		setPreferredSize(new Dimension(470,600));
		GridBagConstraints con = new GridBagConstraints();
		con.anchor = GridBagConstraints.NORTHWEST;
		con.gridx = 0;
		con.gridy = 0;
		
		JPanel topPanel = new JPanel();
		nick = new JLabel(player.getNick());
		role = new JLabel(" ("+player.getRole()+")");
		topPanel.add(nick);
		topPanel.add(role);
		this.add(topPanel,con);

		con.insets = new Insets(20, 0, 0, 0);
		
		con.gridy = 1;
		con.gridx = 0;
		
		em1 = new JLabel("Emotion at stage one:");
		this.add(em1,con);
		
		con.insets = new Insets(3, 0, 0, 0);
		
		con.gridy = 2;
		
		em2 = new JLabel("Emotion at stage two:");
		this.add(em2,con);
		
		con.gridy = 3;
		
		em3 = new JLabel("Emotion at stage three:");
		this.add(em3,con);
		
		con.insets = new Insets(20, 0, 0, 0);
		con.gridy = 4;
		JLabel eqLabel = new JLabel("equipmentList:");
		this.add(eqLabel, con);
		con.insets = new Insets(0, 0, 0, 0);
		
		con.gridy = 5;
		equipmentList = new JList<String>();
		DefaultListModel<String> listModel = new DefaultListModel<String>();
		for(String eq : player.getEquipmentList()){
			listModel.addElement(eq);
		}
		
		equipmentList.setModel(listModel);
		this.add(equipmentList,con);
		
		
		setUpZoneHistoryTab();
		setUpReflectionAnswerTab();
		setUpStatsTabPane();
		
		con.gridx = 0;
		con.gridy = 7;
		con.insets = new Insets(15, 0, 0, 0);
		con.fill = GridBagConstraints.BOTH;
		setUpButtonPanel();
		setUpEmotionData();
		this.add(buttonPanel, con);
		
	}
	
	/**
	 * sets up window-tab which will display both the zone history and the reflection answer submitted by the player
	 */
	public void setUpStatsTabPane(){
		statsTabPane = new JTabbedPane();
		statsTabPane.setAutoscrolls(true);
		
		statsTabPane.add("Zone History",zoneHistoryTab);
		statsTabPane.add("Question answers", reflectionAnswerTab);
		
		GridBagConstraints con = new GridBagConstraints();
		con.insets = new Insets(20, 0, 0, 0);
		con.gridx = 0;
		con.gridy = 6;
		con.fill = GridBagConstraints.BOTH;
		this.add(statsTabPane,con);
	}
	
	/**
	 * Set up the zone history tab.
	 */
	public void setUpZoneHistoryTab(){
		zoneHistoryTab = new JPanel();
		JList<String> historyList = new JList<>();
		historyList.setVisibleRowCount(8);
		DefaultListModel<String> model = new DefaultListModel<>();
		for(String s : player.getZoneHistoryList()){
			model.addElement(s);
		}
		historyList.setModel(model);
		
		JScrollPane scroll = new JScrollPane(historyList);
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
//		zoneHistoryTab.add(historyList);
		zoneHistoryTab.add(scroll);
		
	}
	
	/**
	 * sets up the reflection answers tab
	 */
	public void setUpReflectionAnswerTab(){
		reflectionAnswerTab = new JPanel();
		reflectionAnswerTab.setLayout(new GridBagLayout());
		GridBagConstraints con = new GridBagConstraints();
		con.gridy = 0;
		q1 = new JLabel("How many civilians do you think there were in the game?");
		reflectionAnswerTab.add(q1,con);
		
		con.gridy = 1;
		q2 = new JLabel("How many civilians do you think are alive?");
		reflectionAnswerTab.add(q2,con);

		con.gridy = 2;
		q3 = new JLabel("From 1 to 10, with you role in mind, how well do you think you performed?");
		reflectionAnswerTab.add(q3,con);
		
		con.gridy = 3;
		q4 = new JLabel("From 1 to 10, how well do you think your communication was?");
		reflectionAnswerTab.add(q4,con);
		

		con.gridy = 4;
		q5 = new JLabel("From 1 to 10, how well do you think your item usage was?");
		reflectionAnswerTab.add(q5,con);
		
		con.gridy = 5;
		q6 = new JLabel("From 1 to 10, how well do you think your prioritisation was?");
		reflectionAnswerTab.add(q6,con);
		
		con.gridy = 6;
		q7 = new JLabel("From 1 to 10, how well do you think your decision making was?");
		reflectionAnswerTab.add(q7,con);
		
		con.gridy = 7;
		q8 = new JLabel("From 1 to 10, how well do you think your team communication making was?");
		reflectionAnswerTab.add(q8,con);

		con.gridy = 8;
		q9 = new JLabel("From 1 to 10, how well do you think your team tem usage was?");
		reflectionAnswerTab.add(q9,con);
		
		con.gridy = 9;
		q10 = new JLabel("From 1 to 10, how well do you think your team decision prioritization was?");
		reflectionAnswerTab.add(q10,con);
		
		con.gridy = 10;
		q11 = new JLabel("From 1 to 10, how well do you think your TEAM decision making was?");
		reflectionAnswerTab.add(q11,con);
		
		//gridbadlayout? questions -> answers?
		
	}
	
	/**
	 * sets up the buttom container which holds the backup buttons.
	 */
	public void setUpButtonPanel(){
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		reSendGameDoneMessage = new JButton("re-send game done");
		reSendGameDoneMessage.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				reflectionObserver.requestReflectionQuestionsToBeAsked(nick.getText());
				
			}
		});
		

		buttonPanel.add(reSendGameDoneMessage);
		
		sendOutData = new JButton("Send data to player");
		sendOutData.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				reflectionObserver.requestSendReflectionAndZoneDataToPlayer(nick.getText());
			}
		});
		
		buttonPanel.add(sendOutData);
	}
	

	/**
	 * The function which are called when the player has sucsessfully submitted the reflection questions answers.
	 * This function will update the GUI to represent the answers
	 */
	@Override
	public void reflectionAnswersGiven(ArrayList<String> answers) {
		q1.setText(q1.getText()+"("+ answers.get(0) +")");
		q2.setText(q2.getText()+"("+ answers.get(1) +")");
		q3.setText(q3.getText()+"("+ answers.get(2) +")");
		q4.setText(q4.getText()+"("+ answers.get(3) +")");
		q5.setText(q5.getText()+"("+ answers.get(4) +")");
		q6.setText(q6.getText()+"("+ answers.get(5) +")");
		q7.setText(q7.getText()+"("+ answers.get(6) +")");
		q8.setText(q8.getText()+"("+ answers.get(7) +")");
		q9.setText(q9.getText()+"("+ answers.get(8) +")");
		q10.setText(q10.getText()+"("+ answers.get(9) +")");
		q11.setText(q11.getText()+"("+ answers.get(10) +")");
		reflectionObserver.requestSendReflectionAndZoneDataToPlayer(nick.getText());
		reflectionAnswerTab.revalidate();
		statsTabPane.revalidate();
		this.revalidate();
		
	}
	
	/**
	 * Sets up the text to represent the the emotion answers submitted by the players
	 */
	public void setUpEmotionData(){
		if(player.getReflectionData().getEmotionAnswers().size() > 0)
			em1.setText(em1.getText() +" "+ player.getReflectionData().getEmotionAnswers().get(0));

		if(player.getReflectionData().getEmotionAnswers().size() > 1)
			em2.setText(em2.getText() + " "+ player.getReflectionData().getEmotionAnswers().get(1));
		if(player.getReflectionData().getEmotionAnswers().size() > 2)
			em3.setText(em3.getText() + " "+ player.getReflectionData().getEmotionAnswers().get(2));
		
	}
}
