package models;

import com.google.gson.annotations.Expose;

/**
 * This class represent the civilian, it has the same logic as the client since we need to update it for the recovery functionality
 *
 */
public class Civilian {

	/**
	 * The name of the civilian which will be displayed at the zone GUI
	 */
	@Expose private String name;
	
	/**
	 * The amount of health he has (0 = dead, 100 = max)
	 */
	@Expose private int health;
	
	/**
	 * The amount of panic he has ( 0 = min/super, 100 = max/bad)
	 */
	@Expose private int panic;
	
	/**
	 * A boolean telling if hes alive or not. This is used to check if the object has to be updated when a new game tick occures
	 */
	@Expose private boolean alive;
	
	/**
	 * The initializer for the class
	 * @param name
	 */
	public Civilian(String name) {
		this.name = name;
		health = 80;
		panic = 20;
		alive = true;
	}
	
	/**
	 * A function that will restore health to the civilian
	 * @param amount the amount of health to be restored
	 */
	
	public void heal(int amount){
		health = health + amount;
		if(health > 100)
			health = 100;
	}
	
	/**
	 * The function that will reduce the panic of the civilian
	 * @param amount the amount of panic to be reduced
	 */
	
	public void calm(int amount){
		panic = panic -amount;
		if(panic < 0)
			panic = 0;
	}

	/**
	 * A simple getter
	 * @return returns the name of the civilian
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * A function that increase the panic lvl of the civilian. This is called when a panic wave occure
	 * @param amount the amount of panic 
	 */
	public void increasePanic(int amount){
		panic = panic + amount;
		if(panic > 100){
			panic = 100;
		}
	}

	/**
	 * a simple getter
	 * @return returns wheter the civilian is alive or not
	 */

	public boolean isAlive() {
		return alive;
	}


	/**
	 * A simple setter
	 * @param panic the panic the civilian shall have 
	 */
	public void setPanic(int panic) {
		this.panic = panic;
	}

	/**
	 * A simple 
	 * @return return the panic of the civilian  
	 */

	public int getPanic() {
		return panic;
	}
	
	/**
	 * A function that reduce the life of the civilian. This happens when the civilian has 100 panic and a game tick occured
	 * @param amount the amount of life to be deducted
	 */
	public void reduceLife(int amount){
		health = health - amount;
		if(health <= 0){
			health = 0;
			alive = false;
		}
	}
	
	public String getStats(){
		String stats = name + "  hp: " +health + "  panic: " +panic;
		return stats;
	}
	

}
