package models;

import interfaces.LocationListenerInterace;

import java.util.ArrayList;

/**
 * The class that hold the game state.
 * 
 */
public class Game {
	
	/**
	 * tells how long the interval between panic waves should be in seconds
	 */
	private int panicWaveInterval; 
	
	/**
	 * tells how long the interval between a game tick should be in seconds
	 */
	private int gameTickInterval;
	
	/**
	 * tells how long the game should last in seconds, this is not in use in this prototype version
	 */
	private int gameDuration; 
	
	/**
	 * tells how long the game has lasted so far, this is not in use in this prototype version
	 */
	private int gameTime;
	
	/**
	 * object that should be sent to the client when preparing the game.
	 */
	private Map map;

	/**
	 * A list of all the players who has joined
	 */
	private ArrayList<Player> playerList;
	
	private int pauseLenght;
	
	/**
	 * set up the dummy standard data which will be used for the evaluation.
	 * In this case game ticks occures each 5 sec, and panic waves occures each 45 sec
	 */
	
	private boolean gameDone;
	
	public void setUpDummyData(){
		panicWaveInterval = 45;
		gameTickInterval = 5;
		gameDuration = 64000;
		gameTime= 0;
		pauseLenght = 30000;
		gameDone = false;
		map = new Map();
		map.setUpEvaluationMap();
		playerList = new ArrayList<>();
	}


	/**
	 * A simple getter
	 * @return returns the map
	 */
	public Map getMap() {
		return map;
	}
	
	public ArrayList<Player> getPlayerList(){
		return playerList;
	}
	
	/**
	 * A simple getter
	 * @return the panic wave interval
	 */
	public int getPanicWaveInterval(){
		return panicWaveInterval;
	}
	
	public int getPauseLenght(){
		return pauseLenght;
	}
	
	public boolean isGetDone(){
		return gameDone;
	}
	
	public void setGameState(boolean isOver){
		gameDone = isOver;
	}

	/**
	 * A function which will update the game state by unlocking a zone
	 * @param zoneName the name of the zone where the blockade has been removed
	 * @param blockadeName the name of the blockade which was removed
	 */
	public void unlockBlockadeInZone(String zoneName, String blockadeName){
		map.unlockBlockadeInZone(zoneName, blockadeName);
	}
	
	/**
	 * Add a player which has joined the game
	 * @param nick the nickname of the player
	 */
	public void addPlayer(String nick, String role){
		Player player = new Player(nick);
		player.setRole(role);
		playerList.add(player);
			
	}
	
	/**
	 * Check if the player has already joined this game session
	 * @param nick the nickname we wish to check
	 * @return true if there already exist a player with that nick, else false
	 */
	public boolean playerExist(String nick){
		for(Player pl : playerList){
			if(pl.getNick().equalsIgnoreCase(nick)){
				return true;
			}
		}
		return false;
	}
	/**
	 * Return the equipmentlist of a player, this is done when the recovery function is used
	 * @param nick the nickname of the player
	 * @return the equipmentlist of the player
	 */
	public ArrayList<String> getPlayerEquipmentList(String nick){
		for(Player pl : playerList){
			if(pl.getNick().equalsIgnoreCase(nick)){
				return pl.getEquipmentList();
				
			}
		}
		//should never happend	
		return null;
	}
	
	
	public Player getPlayerByNick(String nick){
		for(Player pl : playerList){
			if(pl.getNick().equalsIgnoreCase(nick)){
				return pl;
				
			}
		}
		//should never happend	
		return null;
	}
	
	public Zone getZoneByName(String name){
		for(Zone zone : map.getZoneList()){
			if(zone.getName().equalsIgnoreCase(name)){
				return zone;
				
			}
		}
		//should never happend	
		return null;
	}
	
	
	/**
	 * Apply an action to a civilian, this is done when a client sends a network msg of this type
	 * @param zoneName the name of the zone the civilian is in
	 * @param civ the name of the civilian the action was used on
	 * @param actionType the type of the action
	 */
	public void applyActionOnCivilian(String zoneName, String civ, String actionType){
		map.applyActionOnCivilian(zoneName, civ, actionType);
	}
	
	
	/**
	 * Apply an equipment directed toward the civilian on the zone
	 * @param zoneName the name of the zone the equipment was used in
	 * @param itemName the name of equipment used
	 */
	public void applyEquipmentOnCivilian(String zoneName, String itemName){
		map.applyEquipmentOnCivilian(zoneName, itemName);
	}
	
	/**
	 * This function is called when a panic wave should occure
	 */
	public void applyPanickWave(){
		map.panicWave();
	}
	
	/**
	 * This function is called when a game tick should occure
	 */
	public void applyGameTick(){
		map.gameTick();
	}
	
	/**
	 * This function is called when a player has used an equipment. The player state is updated for recovery purposes
	 * @param nick the nickname of the player
	 * @param equip the equipment name wich will be removed
	 */
	
	public void removeEquipmentFromPlayerEquipmentList(String nick, String equip){
	//	getPlayerEquipmentList(nick).remove(equip); 
		getPlayerByNick(nick).removeEquipment(equip);
	}
	
	/**
	 * This function is called when a player picks up an equipment. The player state is updated for recovery purposes
	 * @param nick the name of the player
	 * @param equip the name of the equipment to be added
	 */
	
	public void addEquipmentFromPlayerEquipmentList(String nick, String equip){
//		getPlayerEquipmentList(nick).add(equip);
		getPlayerByNick(nick).addEquipment(equip);
	}
	
	public void setReflectionAnswerOnAPlayer(String nick, String answer){
		getPlayerByNick(nick).setReflectionQuestionAnswers(answer);
		//si fra at svarene kan sendes til spilleren.
	}
	
	public void addEmotionAnswerOnAPlayer(String nick, String answer){
		getPlayerByNick(nick).addEmotionAnswerToList(answer);
	}
	
	
	/**
	 * a simple getter
	 * @return returns the amount of civilian still alive
	 */
	public int getNumberOfCiviliansAlive(){
		return map.getCiviliansAlive();
	}
	
	/**
	 * a simple getter
	 * @return return the total amount of civilian in the game, no matter if they are dead or alive
	 */
	public int getNumberOfCiviliansInGame(){
		return map.getCiviliansCount();
	}


	/**
	 * a simple getter
	 * @return return the game tick invervall
	 */

	public int getGameTickInterval() {
		return gameTickInterval;
	}
	
	public void playerEnterZone(String nick, String zoneName){
		Player pl = getPlayerByNick(nick);
		Zone zone = getZoneByName(zoneName);
		if(pl != null && zone != null)
			pl.setZone(zone);
		
	}
	
	
	public void playerLeaveZone(String nick){
		Player pl  = getPlayerByNick(nick);
		if(pl != null)
			pl.setZoneToNull();
	}
	
	public void showPlayerActionUsed(String nick, String msg){
		Player pl = getPlayerByNick(nick);
		if(pl != null)
			pl.testActionUsed(msg);
	}
	
	public void showPlayerUsingActionOnCivilian(String nick,String action, String civ){
		String msg = "Using " + action + " on " + civ;
		showPlayerActionUsed(nick, msg);
		
	}
	
	public void showPlayerUsingEquipmentOnCivilian(String nick, String eq){
		String msg = "Using " +eq;
		showPlayerActionUsed(nick, msg);
	}
	
	public void showPlayerRemovingBlockade(String nick, String eq){
		String msg = "Removing blockade using " + eq;
		showPlayerActionUsed(nick, msg);
	}
	
	public void showItemUsageFinished(String nick, String itemName){
		String msg = "Done using the item " + itemName;
		showPlayerActionUsed(nick, msg);
		
	}
	
	public void showActionUsageFinished(String nick, String civ, String action){
		String msg = "Done using the action " + action + " on " + civ;
		showPlayerActionUsed(nick, msg);
	}
	
	
	public void addZoneEnteredWithTimeStampToAPlayer(String nick,String formated){
		getPlayerByNick(nick).addZoneToHistoryList(formated);
	}
	

	
	
}
