package models;

import java.util.ArrayList;

import com.google.gson.annotations.Expose;


/**
 *is an virtual representation of the game session. This should be sent to the player before the game start so they
 *can set up the same thing. And player should also get notice when something happens.
 */
public class Map {
	/**
	 * A list of zone which is to be used in the game
	 */
	@Expose private ArrayList<Zone> zones; 
	
	/**
	 * this is the function that decide what map to use.
	 */
	public void setUpEvaluationMap(){

//		setUpTaskOneAlternative();
//		setUpTaskTwoAlternativeMapOne();
		setUpTaskTwoAlternativeMapTwo();
	}
	/**
	 * A function which unlock a blockade
	 * @param zoneName the name of the zone the blockade is in
	 * @param blockadeName the name of the blockade which is to be unlocked
	 */
	public void unlockBlockadeInZone(String zoneName, String blockadeName){
		for(Zone zone : zones){
			if(zone.getName().equalsIgnoreCase(zoneName)){
				zone.removeBlockade(blockadeName);
				break;
			}
		}
	}
	
	/**
	 * Apply an action to a civilian, this is done when a client sends a network msg of this type
	 * @param zoneName the name of the zone the civilian is in
	 * @param civ the name of the civilian the action was used on
	 * @param actionType the type of the action
	 */
	public void applyActionOnCivilian(String zoneName, String civ, String actionType){

		for(Zone z : zones){
			if(z.getName().equalsIgnoreCase(zoneName)){
				if(actionType.equalsIgnoreCase("Calming")){
					z.calmCiv(civ, 5);
				}
				else{
					z.healCiv(civ, 5);
				}
					
			}
		}
		
	}
	/**
	 * Apply an equipment directed toward the civilian on the zone
	 * @param zoneName the name of the zone the equipment was used in
	 * @param itemName the name of equipment used
	 */
	public void applyEquipmentOnCivilian(String zoneName, String itemName){
		for(Zone z : zones){
			if(z.getName().equalsIgnoreCase(zoneName)){
				if(itemName.equalsIgnoreCase("medical kit")){
					z.healAllCiv(10);
					break;
				}
				else{
					z.calmAllCiv(10);
					break;
				}
			}
		}
	}
	/**
	 * the function that iterate the zone list and trigger the panic wave function
	 */
	public void panicWave(){
		for(Zone z : zones){
			z.panickWave(10);
		}
	}
	/**
	 * the function that iterate the zone list and trigger the game tickfunction
	 */
	public void gameTick(){
		for(Zone z : zones){
			z.gameTick(10);
		}
	}
	
	
	/**
	 * This sets up a game map, suited for scenario 1
	 */
	public void setUpTaskOne(){
		zones = new ArrayList<Zone>();
		Zone z = new Zone();
		z.setUpDummyData();
		z.setNormalZone(false);
		z.setCord(63.41763 	,10.40306);
		z.setName("safty");
		zones.add(z);
		
		Zone z2 = new Zone();
		z2.setUpDummyData();
		z2.setCord(63.41798 	,10.40270);
		z2.setName("Zone 1");
		z2.addEquipment("fire hose");
		zones.add(z2);
		
		Zone z3 = new Zone();
		z3.setUpDummyData();
		z3.setCord(63.41841 	,10.40245);
		zones.add(z3);
		z3.setName("Zone 2");
		
		Zone z4 = new Zone();
		z4.setUpDummyData();
		z4.setCord(63.41829,10.40326);
		zones.add(z4);
		z4.setName("Zone 3");
		
	}
	
	/**
	 * This set up a game map suited for scenario 2.
	 */
	public void setUpTaskTwo(){
		zones = new ArrayList<Zone>();
		Zone z = new Zone();
		z.setNormalZone(false);
		z.setCord(63.41763 	,10.40306);
		z.setName("safty");
		zones.add(z);
		
		Zone z2 = new Zone();
		z2.setCord(63.41798 	,10.40270);
		z2.setName("Zone 1");
		z2.addBlockade("Fire");
		for(int i = 0; i < 2; i++){
			Civilian civ = new Civilian("Civ" + i);
			z2.addCivilian(civ);
		}
		zones.add(z2);
		
		
		Zone z3 = new Zone();
		z3.setCord(63.41841 	,10.40245);	
		z3.addEquipment("fire hose");
		z3.setName("Zone 2");
		
		for(int i = 0; i < 2; i++){
			Civilian civ = new Civilian("Civ" + i);
			z3.addCivilian(civ);
		}
		
		zones.add(z3);
		
		Zone z4 = new Zone();
		z4.setCord(63.41829,10.40326);
		z4.setName("Zone 3");
		z4.addBlockade("Fire");
		z4.addEquipment("fire hose");
		for(int i = 0; i < 2; i++){
			Civilian civ = new Civilian("Civ" + i);
			z4.addCivilian(civ);
		}
		zones.add(z4);
		
		
		Zone z5 = new Zone();
		z5.setCord(63.41892 ,	10.40298);
		z5.setName("Zone 4");
		z5.addBlockade("Fire");
		for(int i = 0; i < 3; i++){
			Civilian civ = new Civilian("Civ" + i);
			z5.addCivilian(civ);
		}
		zones.add(z5);
		
		Zone z6 = new Zone();
		z6.setCord(63.41906 	,10.40086);
		z6.setName("Zone 5");
		for(int i = 0; i < 8; i++){
			Civilian civ = new Civilian("Civ" + i);
			z6.addCivilian(civ);
		}
		zones.add(z6);
		
		
		Zone z7 = new Zone();
		z7.setCord(63.41948 	,10.40081);
		z7.setName("Zone 6");
		z7.addEquipment("fire hose");
		
		for(int i = 0; i < 10; i++){
			Civilian civ = new Civilian("Civ" + i);
			z7.addCivilian(civ);
		}
		zones.add(z7);
		
		Zone z8 = new Zone();
		z8.setCord(63.41980 	,10.40243);
		z8.setName("Zone 7");
		z8.addBlockade("Fire");
		for(int i = 0; i < 20; i++){
			Civilian civ = new Civilian("Civ" + i);
			z8.addCivilian(civ);
		}
		zones.add(z8);
		
	}
	/**
	 * This sets up a game map, suited for scenario 1. This is the one we used to evaluate with
	 */
	public void setUpTaskOneAlternative(){
		zones = new ArrayList<Zone>();
		Zone z = new Zone();
		z.setUpDummyData();
		z.setNormalZone(false);
		z.setCord(63.41555, 	10.40110);
		z.setName("safty");
		zones.add(z);
		
		Zone z2 = new Zone();
		z2.setUpDummyData();
		z2.setCord(63.41603 ,	10.40079);
		z2.setName("Zone 1");
		z2.addEquipment("fire hose");
		zones.add(z2);
		
		Zone z3 = new Zone();
		z3.setUpDummyData();
		z3.setCord(63.41591 	,10.40254);
		zones.add(z3);
		z3.setName("Zone 2");
		
		Zone z4 = new Zone();
		z4.setUpDummyData();
		z4.setCord(63.41632 ,	10.40222);
		zones.add(z4);
		z4.setName("Zone 3");
		
	}
	
	
	/**
	 * This set up a game map suited for scenario 2. This is the one we used to evaluate with
	 */
	public void setUpTaskTwoAlternativeMapTwo(){
		zones = new ArrayList<Zone>();
		Zone z = new Zone();
		z.setNormalZone(false);
		z.setCord(63.41555, 	10.40110);
//		z.setCord(63.4168320 ,	10.4030673);
		z.setName("Secure camp");
		zones.add(z);
		
		Zone z2 = new Zone();
		z2.setCord(63.41603 ,	10.40079);
		z2.setName("Zone 1");
		z2.addBlockade("Fire");
		for(int i = 0; i < 2; i++){
			Civilian civ = new Civilian("Civ" + i);
			z2.addCivilian(civ);
		}
		zones.add(z2);
		
		
		Zone z3 = new Zone();
		z3.setCord(63.41591 	,10.40254);
		z3.setName("Zone 2");
		z3.addBlockade("jammed");
		for(int i = 0; i < 6; i++){
			Civilian civ = new Civilian("Civ" + i);
			z3.addCivilian(civ);
		}
		
		zones.add(z3);
		
		Zone z4 = new Zone();
		z4.setCord(63.41632 ,	10.40222);
		z4.setName("Zone 3");
		z4.addBlockade("jammed");
		for(int i = 0; i < 3; i++){
			Civilian civ = new Civilian("Civ" + i);
			z4.addCivilian(civ);
		}
		zones.add(z4);
		
		
		Zone z5 = new Zone();
		z5.setCord(63.41674 	,10.40039);
		z5.setName("Zone 4");
		z5.addEquipment("fire hose");
		z5.addBlockade("Fire");
		for(int i = 0; i < 5; i++){
			Civilian civ = new Civilian("Civ" + i);
			z5.addCivilian(civ);
		}
		zones.add(z5);
		
		Zone z6 = new Zone();
//		z6.setCord(63.41714 	,10.40107);
		z6.setCord(63.417122, 10.400721);
		z6.setName("Zone 5");
		z6.addBlockade("Fire");
		for(int i = 0; i < 8; i++){
			Civilian civ = new Civilian("Civ" + i);
			z6.addCivilian(civ);
		}
		zones.add(z6);
		
		
		Zone z7 = new Zone();
		z7.setCord(63.41734 ,	10.39921);
		z7.setName("Zone 6");
		z7.addBlockade("Jammed");
		
		for(int i = 0; i < 9; i++){
			Civilian civ = new Civilian("Civ" + i);
			z7.addCivilian(civ);
		}
		zones.add(z7);
		
		Zone z8 = new Zone();
//		z8.setCord(63.41763 ,	10.40025);
		z8.setCord(63.4168320 ,	10.4030673);
		z8.setName("Zone 7");
		z8.addBlockade("Fire");
		z8.addEquipment("fire hose");
		for(int i = 0; i < 12; i++){
			Civilian civ = new Civilian("Civ" + i);
			z8.addCivilian(civ);
		}
		zones.add(z8);
	}
	
	
	public void setUpTaskTwoAlternativeMapOne(){
		zones = new ArrayList<Zone>();
		Zone z = new Zone();
		z.setNormalZone(false);
		z.setCord(63.41674 	,10.40039);
		z.setName("Secure camp");
		zones.add(z);
		
		Zone z2 = new Zone();
		z2.setCord(63.41603 ,	10.40079);
		z2.setName("Zone 1");
		z2.addBlockade("Fire");
		for(int i = 0; i < 5; i++){
			Civilian civ = new Civilian("Civ" + i);
			z2.addCivilian(civ);
		}
		zones.add(z2);
		
		
		Zone z3 = new Zone();
		z3.setCord(63.41591 	,10.40254);
		z3.setName("Zone 2");
		z3.addBlockade("jammed");
		for(int i = 0; i < 4; i++){
			Civilian civ = new Civilian("Civ" + i);
			z3.addCivilian(civ);
		}
		
		zones.add(z3);
		
		Zone z4 = new Zone();
		z4.setCord(63.41632 ,	10.40222);
		z4.setName("Zone 3");
		z4.addBlockade("jammed");
		for(int i = 0; i < 6; i++){
			Civilian civ = new Civilian("Civ" + i);
			z4.addCivilian(civ);
		}
		zones.add(z4);
		
		
		Zone z5 = new Zone();
		z5.setCord(63.41555, 	10.40110);
		z5.setName("Zone 4");
		z5.addEquipment("fire hose");
		z5.addBlockade("Fire");
		for(int i = 0; i < 7; i++){
			Civilian civ = new Civilian("Civ" + i);
			z5.addCivilian(civ);
		}
		zones.add(z5);
		
		Zone z6 = new Zone();
		z6.setCord(63.417122, 10.400721);
//		z6.setCord(63.417185, 10.402634);
		z6.setName("Zone 5");
		z6.addBlockade("Fire");
		for(int i = 0; i < 5; i++){
			Civilian civ = new Civilian("Civ" + i);
			z6.addCivilian(civ);
		}
		zones.add(z6);
		
		
		Zone z7 = new Zone();
		z7.setCord(63.41734 ,	10.39921);
//		z7.setCord(63.417403,	10.40265);
		z7.setName("Zone 6");
		z7.addBlockade("Jammed");
		
		for(int i = 0; i < 8; i++){
			Civilian civ = new Civilian("Civ" + i);
			z7.addCivilian(civ);
		}
		zones.add(z7);
		
		Zone z8 = new Zone();
		z8.setCord(63.4168674 ,	10.4030174);
		z8.setName("Zone 7");
		z8.addBlockade("Fire");
		z8.addEquipment("fire hose");
		for(int i = 0; i < 12; i++){
			Civilian civ = new Civilian("Civ" + i);
			z8.addCivilian(civ);
		}
		zones.add(z8);
	}


	/**
	 * a simple getter
	 * @return returns the amount of civilian still alive
	 */
	public int getCiviliansAlive(){
		int nr = 0;
		for(Zone zone: zones){
			for(Civilian civ : zone.getCivilianList()){
				if(civ.isAlive() ){
					nr ++;
				}
			}
		}
		
		return nr;
		
	}
	/**
	 * a simple getter
	 * @return return the total amount of civilian in the game, no matter if they are dead or alive
	 */
	public int getCiviliansCount(){
		int nr = 0;
		for(Zone zone: zones){
			nr = nr + zone.getCivilianList().size();
		}
		return nr;
		
	}
	
	public ArrayList<Zone> getZoneList(){
		return zones;
	}
	
}
