package models;

import interfaces.LocationListenerInterace;
import interfaces.PlayerStatsInterface;
import interfaces.ReflectionAnswersInterface;

import java.util.ArrayList;
import java.util.Observer;

import observers.ZoneStats;

/**
 * The class that hold the state of each player. This is used for recovery purposes.
 */
public class Player {

	/**
	 * the nickname of the player
	 */
	private String nick;
	
	/**
	 * The role of the player
	 */
	private String role;
	
	/**
	 * A list of equipment the player has
	 */
	private ArrayList<String> equipmentList;
	
	
	private Zone currentZone;
	private ZoneStats zoneObserver;
	private PlayerStatsInterface playerStatsObserver;
	private LocationListenerInterace zoneEnterObserver;
	
	
//	private ArrayList<String> zoneVisitedWithTimeStamp;
//	
//	private ArrayList<String> reflectionAnswersGiven;
	
	private ReflectionData reflectionData;
	
	private ReflectionAnswersInterface reflectionAnswerObserver;
	
	/**
	 * The contructor for the object
	 * @param nick the name the player shall have
	 */
	public Player(String nick) {
		this.nick = nick;
		this.equipmentList = new ArrayList<>();
		reflectionData = new ReflectionData();
//		zoneVisitedWithTimeStamp = new ArrayList<>();
//		reflectionAnswersGiven = new ArrayList<>();
		
	}
	
	/**
	 * A simple getter
	 * @return returns the nickname
	 */
	public String getNick() {
		return nick;
	}
	
	/**
	 * A simple setter
	 * @param nick set the nickname of the player
	 */
	public void setNick(String nick) {
		this.nick = nick;
	}
	
	/**
	 * A simple getter
	 * @return returns the role of the player
	 */
	public String getRole() {
		return role;
	}
	
	/**
	 * A simple setter, set the role of the player
	 * @param role the role the player shall have
	 */
	public void setRole(String role) {
		this.role = role;
		if(playerStatsObserver != null){
			playerStatsObserver.onRoleSet(role);
		}
	}
	
	public void setReflectionAnswerObserver(ReflectionAnswersInterface ob){
		reflectionAnswerObserver = ob;
	}
	
	
	public void setZoneEnteredObserver(LocationListenerInterace ob){
		zoneEnterObserver = ob;
	}
	
	/**
	 * A simple getter
	 * @return the list of equipment the player has
	 */
	public ArrayList<String> getEquipmentList() {
		return equipmentList;
	}
	
	/**
	 * A simple setter, set the equipmentlist
	 * @param equipmentList the list of equipment the player shall have
	 */
	public void setEquipmentList(ArrayList<String> equipmentList) {
		this.equipmentList = equipmentList;
	}
	

	public void setZone(Zone zone){
		if(zone != currentZone){
			if(currentZone != null){
				setZoneToNull();
			}
				
			currentZone = zone;
			zoneObserver.onEnterZone(zone);
			playerStatsObserver.onZoneEntered(zone.getName());
			zoneEnterObserver.onZoneEnter(nick, currentZone.getName());
			
//			for(Civilian civ : zone.getCivilianList()){
//				System.out.println(civ.getStats());
//			}
		}
	}
	
	public Zone getZone(){
		return currentZone;
	}
	
	public void setZoneToNull(){
		zoneObserver.onLeaveZone();
		playerStatsObserver.onZoneLeft(currentZone.getName());
		currentZone = null;
	}
	
	public void removeEquipment(String equipment){
		equipmentList.remove(equipment);
		playerStatsObserver.onEquipmentUsed(equipment);
	}
	
	public void addEquipment(String equipment){
		equipmentList.add(equipment);
		playerStatsObserver.onEquipmentPickup(equipment);
	}
	
	public void setPlayerStatsObserver(PlayerStatsInterface ob){
		playerStatsObserver = ob;
	}
	
	public void setZoneStatsObserver(ZoneStats ob){
		zoneObserver = ob;
	}
	
	public void testActionUsed(String msg){
		playerStatsObserver.onActionUsed(msg);
	}
	
	public void addZoneToHistoryList(String nameAndTime){
//		zoneVisitedWithTimeStamp.add(nameAndTime);
		reflectionData.addZoneWithTimeStampToList(nameAndTime);
	}
	
	public ArrayList<String> getZoneHistoryList(){
//		return zoneVisitedWithTimeStamp;
		return reflectionData.getZoneVisitedWithTimeStamp();
	}
	
	
	public void setReflectionQuestionAnswers(String formated){
//		String[] answers = formated.split("~");
//		for(String s : answers){
//			reflectionAnswersGiven.add(s);
//		}
		reflectionData.addReflectionAnswersGiven(formated);
		reflectionAnswerObserver.reflectionAnswersGiven(reflectionData.getReflectionAnswersGiven());
		
	}
	
	public void addEmotionAnswerToList(String answer){
		reflectionData.addEmotionAnswerGiven(answer);
	}
	
	public ReflectionData getReflectionData(){
		return reflectionData;
	}
	
//	public String getReflectionQuestionAnswersForamted(){
//		String formated = new String();
//		for(String s : reflectionAnswersGiven){
//			formated += "~" +s;
//		}
//		
//		return formated;
//	}
//	
//	
//	public String getZoneHistoryFormated(){
//		String formated = new String();
//		for(String s : zoneVisitedWithTimeStamp){
//			formated += "~" +s;
//		}
//		return formated;
//	}
	
}
