package models;

import interfaces.CivilianListenerInterface;

import java.util.ArrayList;

import com.google.gson.annotations.Expose;



/**
 * the class that represent a zone
 *
 */
public class Zone {
	
	/**
	 * The name of the zone
	 */
	@Expose private String name;
	
	/**
	 * the longitude of the zone
	 */
	@Expose private double longitude;
	
	/**
	 * the latitde of the zone
	 */
	@Expose private double latitude;
	
	/**
	 * A boolean telling if the zone is locked or not
	 */
	@Expose private boolean locked;
	
	/**
	 * A list of blockade within the zone
	 */
	@Expose private ArrayList<Blockade> blockadeList;
	
	/**
	 * the panic level of the zone
	 */
	@Expose private int panicLevel;
	/**
	 * the list of the civilian 
	 */
	@Expose private ArrayList<Civilian> civilianList;
	
	/**
	 * A boolean used to tell if its a normal zone or a secure camp
	 */
	@Expose private boolean normalZone;
	/**
	 * A list of equipment within the zone
	 */
	@Expose private ArrayList<Equipment> equipmentInZoneList;
	


	/**
	 * The list of equipment which will be sent to the client 
	 */
	@Expose private ArrayList<String> equipmentFromServer;

	/**
	 * The constructor which will initalise needed variable. The zone is set as normal zone as deault
	 */
	
	private ArrayList<CivilianListenerInterface> zoneObersvers;
	
	public Zone() {
		equipmentFromServer = new ArrayList<String>();
		equipmentInZoneList = new ArrayList<Equipment>();
		civilianList = new ArrayList<Civilian>();
		blockadeList = new ArrayList<Blockade>();
		normalZone = true;
		zoneObersvers = new ArrayList<>();
		
	}
	
	/**
	 * Test function allowing us to make custom zones. This is used to set up phase 1 of the evaluation
	 */
	public void setUpDummyData(){
		this.name = "dummy zone";
		this.locked = true;			
		this.latitude = 1;
		this.longitude = 1;
		this.normalZone = true;
		blockadeList = new ArrayList<Blockade>();
		Blockade b = new Blockade("fire");
		blockadeList.add(b);
		Civilian c1 =new Civilian("Jonny");
		Civilian c2 =new Civilian("Siri");
		Civilian c3 =new Civilian("Bob");
		civilianList.add(c1);
		civilianList.add(c2);
		civilianList.add(c3);
		

		
	}
	
	
	
	/**
	 * Set the coordinate of the zone
	 * @param lat the latitude
	 * @param lng the longitude
	 */
	public void setCord(double lat, double lng){
		this.latitude = lat;
		this.longitude = lng;
	}
	
	/**
	 * A simple setter
	 * @param val set if the zone should be normal or not (secure camp).
	 */
	public void setNormalZone(boolean val){
		normalZone = val;
	}
	
	/**
	 * Add equipment to the zone which will be sent to the clients when they download the map
	 * @param name the name of the equipment to be added
	 */
	public void addEquipment(String name){
		equipmentFromServer.add(name);
	}
	
	/**
	 * Remove a blockade from the zone
	 * @param blockadeName the name of the blockade
	 */
	public void removeBlockade(String blockadeName){
		for(Blockade block : blockadeList){
			if(block.getName().equalsIgnoreCase(blockadeName)){
				blockadeList.remove(block);
				break;
			}
		}
		if(blockadeList.size() == 0){
			this.locked = false;
		}
	}

	/**
	 * A simple setter which set the name of the zone
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * A simple getter
	 * @return returns the name of the zone
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * The function that restore health to a civilian
	 * @param civName the name of the civilian
	 * @param amount the amount to be healed
	 */
	
	public void healCiv(String civName, int amount){
		for(Civilian civ : civilianList){
			if(civ.getName().equalsIgnoreCase(civName)){
				civ.heal(amount);
				break;
			}
		}
		notifyObservers();
	}
	
	/**
	 * The function that restore health to all the civilian in the zone
	 * @param amount the amount of health to be restored
	 */
	public void healAllCiv(int amount){
		for(Civilian civ : civilianList){
			civ.heal(amount);
		}
		notifyObservers();
	}
	
	/**
	 * The function that reduce the panic level of civilian
	 * @param civName the name of the civilian
	 * @param amount the amount to be reduced
	 */
	public void calmCiv(String civName, int amount){
		for(Civilian civ : civilianList){
			if(civ.getName().equalsIgnoreCase(civName)){
				civ.calm(amount);
				break;
			}
		}
		notifyObservers();
	}
	
	/**
	 * the function that calm all the civilian
	 * @param amount the amount of panic which will be removed
	 */
	public void calmAllCiv(int amount){
		for(Civilian civ : civilianList){
			civ.calm(amount);
		}
		notifyObservers();
	}

	/**
	 * A simple getter 
	 * @return return the list of civilians within the zone
	 */
	public ArrayList<Civilian> getCivilianList() {
		return civilianList;
	}
	
	public ArrayList<String> getEquipmentInZoneList() {
		return equipmentFromServer;
	}
	
	/**
	 * The function that simulare the panic wave
	 * @param amount the amount of panic to be increased on all the civilian
	 */
	public void panickWave(int amount){
		for(Civilian civ : civilianList){
			civ.increasePanic(10);
		}
		calculatePanicLevel();
		notifyObservers();
	}
	
	/**
	 * A function that update the panic level of the zone by taking the avg panic level on civilian which are alive
	 */
	private void calculatePanicLevel(){
		int max =1;
		for(Civilian civ : civilianList){
			if(civ.isAlive())
				max = max+ civ.getPanic();
		}
		if(civilianList.size() > 0)
			panicLevel = max/civilianList.size();
	}
	
	/**
	 * the function that simulare the game tick.
	 * @param amount the amount of health to be reduced if the civilian panic lvl is 100
	 */
	public void gameTick(int amount){
		for(Civilian civ : civilianList){
			if(civ.isAlive() && civ.getPanic() >= 100){
				civ.reduceLife(amount);
			}
		}
		notifyObservers();
	}
	/**
	 * Add a new cilivian to the one
	 * @param civ the civilian to be added
	 */
	public void addCivilian(Civilian civ){
		civilianList.add(civ);
	}
	
	/**
	 * Add a blockade to the zone
	 * @param name the name of the zone to be added.
	 */
	public void addBlockade(String name){
		Blockade b = new Blockade(name);
		blockadeList.add(b);
	}
	
	public void addObserver(CivilianListenerInterface ob){
		zoneObersvers.add(ob);
	}
	
	public void removeObserver(CivilianListenerInterface ob){
		if(ob != null)
			zoneObersvers.remove(ob);
	}
	
	public void notifyObservers(){
		for (CivilianListenerInterface ob : zoneObersvers) {
			ob.onCiviliansInZoneChanged();
		}
	}
}
