package server;

import game.Main;
import interfaces.PlayerJoinInterface;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import client.PlayerClientThread;
import models.Game;
import models.Map;
import models.Player;

/**
 * The class that handle networking
 *
 */
public class GameServer extends Thread{

	/**
	 * The ServerSocket which is a build in java class that is suited to act as a simple server
	 */
	private ServerSocket serverSocket = null;
	  // The client socket.
	
	/**
	 * The clientsocket which the client will try to connect to
	 */
	private Socket clientSocket = null;
	
	/**
	 * The variable that will limit how many player are allowed to join the server 
	 */
	private final int maxClients = 10;
	
	/**
	 * The list of all the clients that has joined so far
	 */
	private final PlayerClientThread[] threads = new PlayerClientThread[maxClients];
	
	/**
	 * The game object that hold the entire game state
	 */
	private final Game game;
	
	/**
	 * The Gson formater which will turn a object to a set of strings
	 */
	private Gson gson;
	
	private PlayerJoinInterface playerJoinObserver;
	
	
	/**
	 * The constructor that sets up the server which will use port 7474
	 */
	public GameServer( PlayerJoinInterface observer){
		try {
			serverSocket = new ServerSocket(7474);
			System.out.println("Server has started");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		gson = new Gson();
		gson = new GsonBuilder().excludeFieldsWithoutExposeAnnotation().create();
		game = new Game();
		playerJoinObserver = observer;
		game.setUpDummyData();
	}
	
	/**
	 * Start the server and allow players to join the server.
	 */
	public void run(){
		while(true){
		      try {
		          clientSocket = serverSocket.accept();
		          int i = 0;
		          for (i = 0; i < maxClients; i++) {
		        	  System.out.println("threads" +i);
		            if (threads[i] == null) {
		            	PrintStream os = new PrintStream(clientSocket.getOutputStream());
		            	os.println("map|"+gson.toJson(game.getMap()));
		              (threads[i] = new PlayerClientThread(clientSocket, threads,maxClients,game, playerJoinObserver)).start();
		              break;
		            } 
		          }
		          if (i == maxClients) {
		            PrintStream os = new PrintStream(clientSocket.getOutputStream());
		            os.println("Server too busy. Try later.");
		            os.close();
		            clientSocket.close();
		          }
		        } catch (IOException e) {
		          System.out.println(e);
		        }
		}
	}

	/**
	 * A simple getter
	 * @return return the game object
	 */
	public Game getGame(){
		return game;
	}
	
	/**
	 * The function that will send a network message to all the client, stopping the game. (emergency pause, which should not be used)
	 */
	public void pauseGame(){
		
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null) {
            	threads[i].sendPauseMessage();
            }
          }
	}
	
	/**
	 * The function which will send a networkm message and trigger the pause with emotion data gathering.
	 */
	public void pauseGameNormal(){
		
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null) {
            	threads[i].sendNormalPauseMessage(game.getPauseLenght());
            }
          }
	}
	
	/**
	 * The function which will send the message indicating that the game session is done.
	 */
	public void gameFinish(){
		game.setGameState(true);
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null) {
            	threads[i].sendgameFinishedMessage();
            }
          }
	}
	
	/**
	 * Re-send a confirmation that the mode is to be changed from playing to review mode on the game application
	 * @param nick the player which the message are to be sent to.
	 */
	public void reSendGameFinishToASinglePlayer(String nick){
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null && threads[i].getNick().equalsIgnoreCase(nick) ) {
            	threads[i].sendgameFinishedMessage();
            }
          }
	}
	
	/**
	 * Send the reflection answers submitted by the user back to the user
	 * @param nick the nickname which the answers are to be sent to.
	 */
	public void sendReflectionDataToPlayer(String nick){
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null && threads[i].getNick().equalsIgnoreCase(nick) ) {
            	threads[i].sendReflectionData(gson.toJson(game.getPlayerByNick(nick).getReflectionData()));
            }
      }
	}
	
	/* Test function which will send the message to the application which will cause it to draw lines in the mapfragment
	public void drawLines(){		
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null) {
            	threads[i].sendDrawLinedMessage();
            }
          }
	}
	*/
	/**
	 * The fnction that will send a panic wave network message to all the client
	 */
	public void panicWave(){
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null) {
            	threads[i].sendMessage("panicWave|server|paaaaanick!!!");
            }
          }
        game.applyPanickWave();
	}
	
	/**
	 * The function which will send a game tick network message to all the client
	 */
	public void gameTick(){
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null) {
            	threads[i].sendMessage("tick|server|paaaaanick!!!");
            }
          }
        game.applyGameTick();
	}
	/**
	 * This function will send a hearthbeat message to the clients.
	 */
	public void hearthBeat(){
        for (int i = 0; i < maxClients; i++) {
            if (threads[i] != null) {
            	threads[i].sendMessage("hearthBeat|hearthBeat|hearthBeat");
            }
          }
	}

}
